package LARRD::config;

use strict;
use FileHandle;
use Exporter;

use vars qw($VERSION);
( $VERSION ) = '$Revision$' =~ /\$Revision:\s+([^\s]+)/;

use vars qw (@ISA @EXPORT @EXPORT_OK);

@ISA = qw(Exporter);

@EXPORT = qw(  );

@EXPORT_OK = qw( @DATALIST @LARRDDISABLE $graphwidth $graphheight );

###############################
# USER DEFINED VARIABLES
# Try to autodetect as much as possible.
###############################

our $graphwidth = "576";
our $graphheight = "200";

$ENV{'BBLOGSTATUS'} = defined($ENV{'BBLOGSTATUS'}) ? $ENV{'BBLOGSTATUS'} : 'STATIC';
$ENV{'BBHOME'} = defined($ENV{'BBHOME'}) ? $ENV{'BBHOME'} : 
  -d '/usr/local/bb/bb' ? '/usr/local/bb/bb' : 
  -d '/usr/local/bb' ? '/usr/local/bb' :
  -d '/export/home/bb/bb' ? '/export/home/bb/bb' :
  -d '/home/bb/bb' ? '/home/bb/bb' :
  -d '/opt/bb' ? '/opt/bb' :
  die("Big Brother home directory BBHOME not found");
$ENV{'BBVAR'} = defined($ENV{'BBVAR'}) ? $ENV{'BBVAR'} :
  -d '/usr/local/bb/bbvar' ? '/usr/local/bb/bbvar' :
  -d '/usr/local/bbvar' ? '/usr/local/bbvar' :
  -d '/export/home/bb/bbvar' ? '/export/home/bb/bbvar' :
  -d '/home/bb/bbvar' ? '/home/bb/bbvar' :
  -d '/var/opt/bbvar' ? '/var/opt/bbvar' :
  die("Big Brother var directory BBVAR not found");
$ENV{'BBLOGS'} = "$ENV{'BBVAR'}/logs";
$ENV{'BBWEB'} = defined($ENV{'BBWEB'}) ? $ENV{'BBWEB'} : '/bb';
$ENV{'CGIBINURL'} = defined($ENV{'CGIBINURL'}) ? $ENV{'CGIBINURL'} : '/cgi-bin' ;

# For ignoring RRD files that are too old
$ENV{'PURPLEDELAY'} = defined($ENV{'PURPLEDELAY'}) ? $ENV{'PURPLEDELAY'} : 30;

# Base directory of rrdtool
$ENV{'RRDDIR'}="/usr";

# Base directory of larrd
$ENV{'LARRDHOME'}="/usr/local/larrd";

# Which datasets should I collect/graph/html for?
#  la = Load Average, Users, Processes
#  disk = Disk Utilization 
#  bbnet = TCP Connect Times and Ping Times
#  vmstat = vmstat statistics (requires bf-larrd.sh)
#  apache =
#  bind =
#  iostat =
#  netstat =
#  sendmail =
#  memory = Memory Utilization (requires bb-memory.sh)
#  files = Graphs file sizes, requires bb-files from deadcat.net
#  ftpusers = Graphs number of connected FTP users. Works with BB's FTP check
#             but requires that the server gives that information out
#             for anonymous logins.

our @DATALIST=("la","procs","users","apache","disk","inode","bbnet","vmstat","iostat","netstat","sendmail","memory","mailq","temperature","files","ftpusers");

# Name of column in BB
$ENV{'LARRDCOLNAME'}="larrd";

# Location of bb-hostsvc.sh for STATIC compliance
# needed if BBLOGSTATUS=STATIC
$ENV{'LARRDHOSTSVC'} = 
  -x "/var/www/cgi-bin/bb-hostsvc.sh" ? "/var/www/cgi-bin/bb-hostsvc.sh" :
  -x "/usr/local/www/cgi-bin/bb/bb-hostsvc.sh" ? "/usr/local/www/cgi-bin/bb/bb-hostsvc.sh" :
  die("Big Brother Display Script bb-hostsvc.sh not found or not executable");


# NOTICE: We deprecated for it checks for DATA then it checks for STATUS
# The bottom feeders sets the method.  This allows for granularity.
#
# How does bf-larrd.sh send data back to BBDISP?
# DATA if BB compiled with -DDATAMSG
# STATUS if BB is not
# AUTO if you want butter to try to read BigBrother's Makefile and figure it out
# $ENV{'LARRDCOMM'}="STATUS";
# $ENV{'LARRDCOMM'}="DATA";
$ENV{'LARRDCOMM'}="DATA";

# Hosts you do not want a larrd HTML dot, data is still trended
our @LARRDDISABLE=("");

# LOGGING

# Messages that will prevent larrd from functiong properly
$ENV{'ERROR'} = defined($ENV{'ERROR'}) ? $ENV{'ERROR'} : 1;
# Messages that tell you what larrd has done to the system
$ENV{'STATUS'} = defined($ENV{'STATUS'}) ? $ENV{'STATUS'} : 1;

# Messages that will not prevent larrd from functioning properly (noisy)
$ENV{'WARN'} = defined($ENV{'WARN'}) ? $ENV{'WARN'} : 0;
# Messages that tell you a little about what larrd is doing (very noisy)
$ENV{'DEBUG'} = defined($ENV{'DEBUG'}) ? $ENV{'DEBUG'} : 0;

###############################
# END USER DEFINED VARIABLES
###############################


$ENV{'RRDPM'}="$ENV{'RRDDIR'}/lib/perl";

$ENV{'RRDTOOL'}="$ENV{'RRDDIR'}/bin/rrdtool";

# Where should module logs go?
# But the CGI which is run as the web server user can't use this anyways
$ENV{'LARRDLOG'}="$ENV{'BBHOME'}/LARRDOUT";

# Location of temporary web graphs
$ENV{'LARRDTMP'}="/tmp/$ENV{'LARRDCOLNAME'}";

# Where should the RRD's be kept?
# pronounced birds
$ENV{'BBRRDS'}="$ENV{'BBVAR'}/rrd";

# what RRA's should be made for each RRD?
# "daily" 5min avg last 48hr
# "weekly" 30min avg last 12days
# "monthly 2hr avg last 48days
# "yearly" 24hr avg last 576days
$ENV{'RRAS'}="RRA:AVERAGE:0.5:1:576 RRA:AVERAGE:0.5:6:576 RRA:AVERAGE:0.5:24:576 RRA:AVERAGE:0.5:288:576";


# If LARRDCOMM is set to AUTO, read BigBrother's Makefile and figure out
# whether or not BigBrother was compiled with -DDATA.
if ($ENV{'LARRDCOMM'} eq 'AUTO') {
  my $infh = new FileHandle;
  if ($infh->open("< $ENV{'BBHOME'}/src/Makefile")) {
    while (my $line = $infh->getline()) {
      if ($line =~ /^CFLAGS/) {
        if ($line =~ /-DDATA/) {
          $ENV{'LARRDCOMM'} = 'DATA';
        } else {
          $ENV{'LARRDCOMM'} = 'STATUS';
        }
        last;
      }
    }
    $infh->close;
    if ($ENV{'LARRDCOMM'} eq 'AUTO') {
      die("Unable to determine whether or not BigBrother was compiled with -DDATA");
    }
  } else {
    die("Unable to open BigBrother's Makefile to determine whether or not BigBrother was compiled with -DDATA: $!");
  }
}


1;
