# Graph configuration info in a file equally accessible by both
# larrd-html.pl and larrd-grapher.cgi

package LARRD::graphinfo;

use strict;
use Exporter;

use vars qw (@ISA @EXPORT @EXPORT_OK);

@ISA = qw(Exporter);

@EXPORT = qw(  );

@EXPORT_OK = qw(
  %start_secs
  %timespan_names
  %graph_titles
  %graph_yaxes
  %graph_lines
  %graph_dsources
  color
  get_dsnames
);

# For padding in disk related graphs
my $longestdiskname = '/vol/vol3/local/sun4'; # Longest I've seen
# For padding in temperature graphs
my $longesttempname = 'RSP(7) Hotpoint';

# These don't change for each service
our %start_secs = (
  'minute' => 'e-8h',
  'hourly' => 'e-48h',
  'daily' => 'e-12d',
  'weekly' => 'e-48d',
  'monthly' => 'e-576d',
);

# These don't change for each service either, formerly known as 'legend'
our %timespan_names = (
  'minute' => 'Last 8 Hours',
  'hourly' => 'Last 48 Hours',
  'daily' => 'Last 12 Days',
  'weekly' => 'Last 48 Days',
  'monthly' => 'Last 576 Days',
);

# These don't change for each service either, and in the original
# they're only used once.
our %average_names = (
  'minute' => '5 Minute average',
  'hourly' => '5 Minute average',
  'daily' => '30 Minute Average',
  'weekly' => '2 Hour Average',
  'monthly' => '1 Day Average',
);


our %graph_yaxes = (
  'mailq' => 'Queue Size: #',
  'mailq0' => 'Queue Size: Bytes',
  'la' => 'Load Average',
  'la0' => 'Percent CPU',
  'inode' => '% Full',
  'disk' => '% Full',
  'disk1' => 'Bytes',
  'bbnet' => "Seconds",
  'vmstat' => "% Load",
  'vmstat3' => "Average Events/s",
  'vmstat4' => "Average B/s",
  'vmstat5' => "Average blocks/s",
  'vmstat6' => "Bytes",
  'vmstat7' => "Events/s",
  'vmstat8' => "Bytes/s",
  'vmstat9' => "Pages Scanned/s",
  'vmstat0' => "System Calls/s",
  'memory' => "% Full",
  'bind' => "Stats",
  'apache' => "Accesses/min",
  'apache1' => "Bits per second",
  'apache2' => "Servers",
  'iostat' => "Seconds",
  'netstat2' => "Connections/s",
  'netstat1' => "packets/s",
  'netstat' => "bits/second",
  'netstat3' => "packets/second",
  'sendmail' => 'Stats',
  'procs' => 'Processes',
  'users' => "Users",
  'ftpusers' => "FTP Users",
  'temperature' => 'Degrees Celsius',
  'files' => 'Bytes',
);

# Everything but lines
our(%graph_titles) = (
  'mailq' => 'Mail Queue Length',
  'mailq0' => 'Mail Queue Space',
  'la' => 'Load Average',
  'la0' => 'Processor Time',
  'inode' => 'Disk Inode Utilization',
  'disk' => 'Disk Space Percentage Utilization',
  'disk1' => 'Disk Space Volume Utilization',
  'bbnet' => 'Service Response Times',
  'vmstat' => 'CPU Utilization',
  'vmstat3' => 'Events',
  'vmstat4' => 'Swap Traffic',
  'vmstat5' => 'Block Device Traffic',
  'vmstat6' => 'Available Memory',
  'vmstat7' => 'Memory Activity',
  'vmstat8' => 'Paging Activity',
  'vmstat9' => 'RAM Scan Rate',
  'vmstat0' => 'System Calls',
  'memory' => 'Memory Utilization',
  'bind' => 'BIND Utilization',
  'apache' => 'Apache Accesses',
  'apache1' => 'Apache Traffic',
  'apache2' => 'Apache Server Utilization',
  'iostat' => 'Disk Average Service Time',
  'netstat2' => 'TCP Connections',
  'netstat1' => 'UDP Packet Traffic',
  'netstat' => 'TCP Traffic',
  'netstat3' => 'TCP Packet Traffic',
  'sendmail' => 'Sendmail Utilization',
  'procs' => '# of Processes',
  'users' => '# of Users',
  'ftpusers' => '# of FTP Users',
  'temperature' => 'Temperature',
  'files' => 'File Sizes',
);



our(%graph_lines) = (
  mailq => sub {
      my($RRD,$graph) = @_;
      return [
        qq{-l 0},
        qq{-X 0},
        qq{-Y},
        qq{DEF:mailq=$RRD:mailq:AVERAGE},
        qq{LINE2:mailq#CC0000:Queue Length},
        qq{COMMENT:\\n},
        qq{GPRINT:mailq:LAST:Current \\: %0.0lf messages\\n},
        qq{GPRINT:mailq:MIN:Minimum \\: %0.0lf messages\\n},
        qq{GPRINT:mailq:AVERAGE:Average \\: %lf messages\\n},
        qq{GPRINT:mailq:MAX:Maximum \\: %0.0lf messages\\n},
      ]
    },
  mailq0 => sub {
      my($RRD,$graph) = @_;
      return [
        qq{-b 1024},
        qq{-l 0},
        qq{-Y},
        qq{DEF:mailspace=$RRD:mailspace:AVERAGE},
        qq{AREA:mailspace#00CC00:Queue Space},
        qq{COMMENT:\\n},
        qq{GPRINT:mailspace:LAST:Current \\: %3.2lf%sB\\n},
        qq{GPRINT:mailspace:MIN:Minimum \\: %3.2lf%sB\\n},
        qq{GPRINT:mailspace:AVERAGE:Average \\: %3.2lf%sB\\n},
        qq{GPRINT:mailspace:MAX:Maximum \\: %3.2lf%sB\\n},
      ]
    },
  la => sub { 
      my($RRD,$graph) = @_;

      return [
         qq{-l 0},
         qq{-X 0},
         qq{-Y},
         qq{DEF:avg=$RRD:la:AVERAGE},
         qq{CDEF:la=avg,100,/},
         qq{AREA:la#00CC00:Load Average $average_names{$graph}},
         qq{COMMENT:\\n},
         qq{GPRINT:la:LAST:Current \\: %3.2lf\\n},
         qq{GPRINT:la:MIN:Minimum \\: %3.2lf\\n},
         qq{GPRINT:la:AVERAGE:Average \\: %lf\\n},
         qq{GPRINT:la:MAX:Maximum \\: %3.2lf\\n},
      ];
    },
  la0 => sub {
      my($RRD,$graph) = @_;

      return [
         qq{-l 0},
         qq{-X 0},
         qq{-Y},
         qq{DEF:proctime=$RRD:ProcessorTime:AVERAGE},
         qq{LINE2:proctime#00CC00:Processor Time $average_names{$graph}},
         qq{COMMENT:\\n},
         qq{GPRINT:proctime:LAST:Current \\: %3.2lf%%\\n},
         qq{GPRINT:proctime:MIN:Minimum \\: %3.2lf%%\\n},
         qq{GPRINT:proctime:AVERAGE:Average \\: %lf%%\\n},
         qq{GPRINT:proctime:MAX:Maximum \\: %3.2lf%%\\n},
      ];
    },
  temperature => sub {
      my($RRD,$graph,$p,$color) = @_;

      my($part) = $RRD;
      $part =~ s/^.*temperature,/,/;
      $part =~ s/\.rrd//;
      $part =~ s/\,//g;
      $part =~ s/_/ /g;
      my $padding = '';
      if (length($part) <= length($longesttempname)) {
        $padding = ' ' x (length($longesttempname) - length($part));
      }

      # Try with no upper or lower limits for now....

      return [
        qq{DEF:$p=$RRD:temperature:AVERAGE},
        qq{LINE2:$p#$color:$part},
        qq{GPRINT:$p:LAST:$padding cur\\:%3.1lfC},
        qq{GPRINT:$p:MIN:min\\:%3.1lfC},
        qq{GPRINT:$p:AVERAGE:avg\\:%3.1lfC},
        qq{GPRINT:$p:MAX:max\\:%3.1lfC},
        qq{COMMENT:\\n},
      ]
    },
  inode => sub {
      my($RRD,$graph,$p,$color) = @_;

      my($part) = $RRD;
      $part =~ s/^.*inode,/,/;
      $part =~ s/\.rrd//;
      $part =~ s/\,/\//g;
      $part = '/' if $part eq '/root';
      my $padding = '';
      if (length($part) <= length($longestdiskname)) {
        $padding = ' ' x (length($longestdiskname) - length($part));
      }

      return [
        qq{-u 100},qq{-l 0},
        qq{DEF:$p=$RRD:pct:AVERAGE},
        qq{LINE2:$p#$color:$part},
        qq{GPRINT:$p:LAST:$padding cur\\:%3.0lf%%},
        qq{GPRINT:$p:MIN:min\\:%3.0lf%%},
        qq{GPRINT:$p:AVERAGE:avg\\:%3.0lf%%},
        qq{GPRINT:$p:MAX:max\\:%3.0lf%%},
        qq{COMMENT:\\n},
      ]
    },
  disk => sub {
      my($RRD,$graph,$p,$color) = @_;

      my($part) = $RRD;
      $part =~ s/^.*disk,/,/;
      $part =~ s/^.*diskswap\.rrd/swap/;    # swap space from ext/dspace.sh
      $part =~ s/\.rrd//;
      $part =~ s/\,/\//g;
      $part = '/' if $part eq '/root';
      my $padding = '';
      if (length($part) <= length($longestdiskname)) {
        $padding = ' ' x (length($longestdiskname) - length($part));
      }

      return [
        qq{-u 100},qq{-l 0},
        qq{DEF:$p=$RRD:pct:AVERAGE},
        qq{LINE2:$p#$color:$part},
        qq{GPRINT:$p:LAST:$padding cur\\:%3.0lf%%},
        qq{GPRINT:$p:MIN:min\\:%3.0lf%%},
        qq{GPRINT:$p:AVERAGE:avg\\:%3.0lf%%},
        qq{GPRINT:$p:MAX:max\\:%3.0lf%%},
        qq{COMMENT:\\n},
      ]
    },
  disk1 => sub {
      my($RRD,$graph,$p,$color) = @_;

      my($part) = $RRD;
      $part =~ s/^.*disk,/,/;
      $part =~ s/^.*diskswap\.rrd/swap/;    # swap space from ext/dspace.sh
      $part =~ s/\.rrd//;
      $part =~ s/\,/\//g;
      $part = '/' if $part eq '/root';
      my $padding = '';
      if (length($part) <= length($longestdiskname)) {
        $padding = ' ' x (length($longestdiskname) - length($part));
      }

      return [
        qq{-b 1024},
        qq{-l 0},
        qq{DEF:${p}D=$RRD:used:AVERAGE},
        qq{CDEF:${p}KB=${p}D,1024,*},
        qq{LINE2:${p}KB#$color:$part},
        qq{GPRINT:${p}KB:LAST:$padding cur\\:%4.2lf%sB},
        qq{GPRINT:${p}KB:MIN:min\\:%4.2lf%sB},
        qq{GPRINT:${p}KB:AVERAGE:avg\\:%4.2lf%sB},
        qq{GPRINT:${p}KB:MAX:max\\:%4.2lf%sB},
        qq{COMMENT:\\n},
      ]
    },
  bbnet => sub {
      my($RRD,$graph,$p,$color) = @_;

      $RRD =~ s/:/\\:/g;

      my($part) = $RRD;
      $part =~ s/^.*bbnet./,/;
      $part =~ s/\.rrd//;
      $part =~ s/\,/\//g;
      $part =~ s/^\///g;

      # Pray that the string "http." isn't in the hostname
      $part =~ s/http\.(\S+)$/http\\:\/\/$1/g;

      return [
        qq{-l 0},
        qq{DEF:$p=$RRD:sec:AVERAGE},
        qq{LINE2:$p#$color:$part\\n},
        qq{GPRINT:$p:LAST:  cur\\:%3.2lf%ss},
        qq{GPRINT:$p:MIN:min\\:%3.2lf%ss},
        qq{GPRINT:$p:AVERAGE:avg\\:%3.2lf%ss},
        qq{GPRINT:$p:MAX:max\\:%3.2lf%ss},
        qq{COMMENT:\\n},
      ]
    },
  vmstat => sub {
      my($RRD,$graph,$p,$color) = @_;

      return [
        qq{-u 100},qq{-r},qq{-l 0},
        qq{DEF:cpu_idl=$RRD:cpu_idl:AVERAGE},
        qq{DEF:cpu_usr=$RRD:cpu_usr:AVERAGE},
        qq{DEF:cpu_sys=$RRD:cpu_sys:AVERAGE},
        qq{AREA:cpu_sys#FF0000:System\\n},
        qq{GPRINT:cpu_sys:LAST:  cur\\:%3.2lf%%},
        qq{GPRINT:cpu_sys:MIN:min\\:%3.2lf%%},
        qq{GPRINT:cpu_sys:AVERAGE:avg\\:%3.2lf%%},
        qq{GPRINT:cpu_sys:MAX:max\\:%3.2lf%%},
        qq{COMMENT:\\n},
        qq{STACK:cpu_usr#FFFF00:User\\n},
        qq{GPRINT:cpu_usr:LAST:  cur\\:%3.2lf%%},
        qq{GPRINT:cpu_usr:MIN:min\\:%3.2lf%%},
        qq{GPRINT:cpu_usr:AVERAGE:avg\\:%3.2lf%%},
        qq{GPRINT:cpu_usr:MAX:max\\:%3.2lf%%},
        qq{COMMENT:\\n},
        qq{STACK:cpu_idl#00FF00:Idle\\n},
        qq{GPRINT:cpu_idl:LAST:  cur\\:%3.2lf%%},
        qq{GPRINT:cpu_idl:MIN:min\\:%3.2lf%%},
        qq{GPRINT:cpu_idl:AVERAGE:avg\\:%3.2lf%%},
        qq{GPRINT:cpu_idl:MAX:max\\:%3.2lf%%},
        qq{COMMENT:\\n},
      ]
    },
  vmstat3 => sub {
      my($RRD,$graph,$p,$color) = @_;

      return [
        qq{-l 0},
        qq{-X 0},
        qq{-Y},
        qq{DEF:int=$RRD:cpu_int:AVERAGE},
        qq{DEF:csw=$RRD:cpu_csw:AVERAGE},
        qq{LINE2:int#0000ff:Interrupts/sec},
        qq{COMMENT:\\n},
        qq{GPRINT:int:LAST:  cur\\:%0.0lf},
        qq{GPRINT:int:MIN:min\\:%0.0lf},
        qq{GPRINT:int:AVERAGE:avg\\:%0.0lf},
        qq{GPRINT:int:MAX:max\\:%0.0lf},
        qq{COMMENT:\\n},
        qq{LINE2:csw#00FF00:Context Switches/sec},
        qq{COMMENT:\\n},
        qq{GPRINT:csw:LAST:  cur\\:%0.0lf},
        qq{GPRINT:csw:MIN:min\\:%0.0lf},
        qq{GPRINT:csw:AVERAGE:avg\\:%0.0lf},
        qq{GPRINT:csw:MAX:max\\:%0.0lf},
        qq{COMMENT:\\n},
      ]
    },
  vmstat4 => sub {
      my($RRD,$graph,$p,$color) = @_;

      return [
        qq{-b 1024},
        qq{-l 0},
        qq{DEF:si=$RRD:mem_si:AVERAGE},
        qq{CDEF:bsi=si,1024,*},
        qq{DEF:so=$RRD:mem_so:AVERAGE},
        qq{CDEF:bso=so,1024,*},
        qq{LINE2:bsi#0000ff:Memory Swapped from Disk},
        qq{COMMENT:\\n},
        qq{GPRINT:bsi:LAST:  cur\\:%3.2lf%sB/s},
        qq{GPRINT:bsi:MIN:min\\:%3.2lf%sB/s},
        qq{GPRINT:bsi:AVERAGE:avg\\:%3.2lf%sB/s},
        qq{GPRINT:bsi:MAX:max\\:%3.2lf%sB/s},
        qq{COMMENT:\\n},
        qq{LINE2:bso#00FF00:Memory Swapped to Disk},
        qq{COMMENT:\\n},
        qq{GPRINT:bso:LAST:  cur\\:%3.2lf%sB/s},
        qq{GPRINT:bso:MIN:min\\:%3.2lf%sB/s},
        qq{GPRINT:bso:AVERAGE:avg\\:%3.2lf%sB/s},
        qq{GPRINT:bso:MAX:max\\:%3.2lf%sB/s},
        qq{COMMENT:\\n},
      ]
    },
  vmstat5 => sub {
      my($RRD,$graph,$p,$color) = @_;

      return [
        qq{-l 0},
        qq{-X 0},
        qq{-Y},
        qq{DEF:bi=$RRD:dsk_bi:AVERAGE},
        qq{DEF:bo=$RRD:dsk_bo:AVERAGE},
        qq{LINE2:bi#0000FF:Blocks sent to a block device},
        qq{COMMENT:\\n},
        qq{GPRINT:bi:LAST:  cur\\:%3.2lf Blocks/s},
        qq{GPRINT:bi:MIN:min\\:%3.2lf Blocks/s},
        qq{GPRINT:bi:AVERAGE:avg\\:%3.2lf Blocks/s},
        qq{GPRINT:bi:MAX:max\\:%3.2lf Blocks/s},
        qq{COMMENT:\\n},
        qq{LINE2:bo#00FF00:Blocks received from a block device},
        qq{COMMENT:\\n},
        qq{GPRINT:bo:LAST:  cur\\:%3.2lf Blocks/s},
        qq{GPRINT:bo:MIN:min\\:%3.2lf Blocks/s},
        qq{GPRINT:bo:AVERAGE:avg\\:%3.2lf Blocks/s},
        qq{GPRINT:bo:MAX:max\\:%3.2lf Blocks/s},
        qq{COMMENT:\\n},
      ]
    },
  vmstat6 => sub {
      my($RRD,$graph,$p,$color) = @_;

      return [
        qq{-b 1024},
        qq{-l 0},
        qq{DEF:free=$RRD:mem_free:AVERAGE},
        qq{CDEF:fr=free,1024,*},
        qq{DEF:swap=$RRD:mem_swap:AVERAGE},
        qq{CDEF:sw=swap,1024,*},
        qq{LINE2:fr#0000ff:Size of the free list},
        qq{COMMENT:\\n},
        qq{GPRINT:fr:LAST:  cur\\:%3.2lf%sB},
        qq{GPRINT:fr:MIN:min\\:%3.2lf%sB},
        qq{GPRINT:fr:AVERAGE:avg\\:%3.2lf%sB},
        qq{GPRINT:fr:MAX:max\\:%3.2lf%sB},
        qq{COMMENT:\\n},
        qq{LINE2:sw#00FF00:Swap space currently available},
        qq{COMMENT:\\n},
        qq{GPRINT:sw:LAST:  cur\\:%3.2lf%sB},
        qq{GPRINT:sw:MIN:min\\:%3.2lf%sB},
        qq{GPRINT:sw:AVERAGE:avg\\:%3.2lf%sB},
        qq{GPRINT:sw:MAX:max\\:%3.2lf%sB},
        qq{COMMENT:\\n},
      ]
    },
  vmstat7 => sub {
      my($RRD,$graph,$p,$color) = @_;

      return [
        qq{-l 0},
        qq{DEF:re=$RRD:mem_re:AVERAGE},
        qq{DEF:mf=$RRD:mem_mf:AVERAGE},
        qq{LINE2:re#0000ff:Page reclaims},
        qq{COMMENT:\\n},
        qq{GPRINT:re:LAST:  cur\\:%0.0lf events/s},
        qq{GPRINT:re:MIN:min\\:%0.0lf events/s},
        qq{GPRINT:re:AVERAGE:avg\\:%0.0lf events/s},
        qq{GPRINT:re:MAX:max\\:%0.0lf events/s},
        qq{COMMENT:\\n},
        qq{LINE2:mf#00FF00:Minor faults},
        qq{COMMENT:\\n},
        qq{GPRINT:mf:LAST:  cur\\:%0.0lf events/s},
        qq{GPRINT:mf:MIN:min\\:%0.0lf events/s},
        qq{GPRINT:mf:AVERAGE:avg\\:%0.0lf events/s},
        qq{GPRINT:mf:MAX:max\\:%0.0lf events/s},
        qq{COMMENT:\\n},
      ]
    },
  vmstat8 => sub {
      my($RRD,$graph,$p,$color) = @_;

      return [
        qq{-b 1024},
        qq{-l 0},
        qq{DEF:pi=$RRD:mem_pi:AVERAGE},
        qq{CDEF:rpi=pi,1024,*},
        qq{DEF:po=$RRD:mem_po:AVERAGE},
        qq{CDEF:rpo=po,1024,*},
        qq{LINE2:rpi#0000ff:Paged in},
        qq{COMMENT:\\n},
        qq{GPRINT:rpi:LAST:  cur\\:%3.2lf%sB/s},
        qq{GPRINT:rpi:MIN:min\\:%3.2lf%sB/s},
        qq{GPRINT:rpi:AVERAGE:avg\\:%3.2lf%sB/s},
        qq{GPRINT:rpi:MAX:max\\:%3.2lf%sB/s},
        qq{COMMENT:\\n},
        qq{LINE2:rpo#00FF00:Paged out},
        qq{COMMENT:\\n},
        qq{GPRINT:rpo:LAST:  cur\\:%3.2lf%sB/s},
        qq{GPRINT:rpo:MIN:min\\:%3.2lf%sB/s},
        qq{GPRINT:rpo:AVERAGE:avg\\:%3.2lf%sB/s},
        qq{GPRINT:rpo:MAX:max\\:%3.2lf%sB/s},
        qq{COMMENT:\\n},
      ]
    },
  vmstat9 => sub {
      my($RRD,$graph,$p,$color) = @_;

      return [
        qq{-l 0},
        qq{DEF:sr=$RRD:sr:AVERAGE},
        qq{AREA:sr#00CC00:Scan Rate\\n},
        qq{GPRINT:sr:LAST:Current \\: %5.3lf pages/s\\n},
        qq{GPRINT:sr:MIN:Minimum \\: %5.3lf pages/s\\n},
        qq{GPRINT:sr:AVERAGE:Average \\: %5.3lf pages/s\\n},
        qq{GPRINT:sr:MAX:Maximum \\: %5.3lf pages/s\\n},
      ]
    },
  vmstat0 => sub {
      my($RRD,$graph,$p,$color) = @_;

      return [
        qq{-l 0},
        qq{DEF:cpu_syc=$RRD:cpu_syc:AVERAGE},
        qq{AREA:cpu_syc#00CC00:Sytem Calls},
        qq{COMMENT:\\n},
        qq{GPRINT:cpu_syc:LAST:Current \\: %5.2lf calls/s\\n},
        qq{GPRINT:cpu_syc:MIN:Minimum \\: %5.2lf calls/s\\n},
        qq{GPRINT:cpu_syc:AVERAGE:Average \\: %5.2lf calls/s\\n},
        qq{GPRINT:cpu_syc:MAX:Maximum \\: %5.2lf calls/s\\n},
      ]
    },
  memory => sub {
      my($RRD,$graph,$p,$color) = @_;

      my($part) = $RRD;
      $part =~ s/\,/\./g;
      $part =~ s/^.*memory\.//;
      $part =~ s/\.rrd//;

      return [
        qq{-b 1024},
        qq{-l 0},
        qq{DEF:$p=$RRD:realmempct:AVERAGE},
        qq{LINE2:$p#$color:$part\\n},
        qq{GPRINT:$p:LAST:cur\\:%3.2lf%%},
        qq{GPRINT:$p:MAX:max\\:%3.2lf%%},
        qq{GPRINT:$p:MIN:min\\:%3.2lf%%},
        qq{GPRINT:$p:AVERAGE:avg\\:%3.2lf%%\\n},
      ]
    },
  bind => sub {
      my($RRD,$graph,$p,$color) = @_;

      return [
        qq{-l 0},
        qq{DEF:A=$RRD:A:AVERAGE},
        qq{DEF:ANY=$RRD:ANY:AVERAGE},
        qq{DEF:AXFR=$RRD:AXFR:AVERAGE},
        qq{DEF:MX=$RRD:MX:AVERAGE},
        qq{DEF:O=$RRD:O:AVERAGE},
        qq{DEF:PTR=$RRD:PTR:AVERAGE},
        qq{DEF:SOA=$RRD:SOA:AVERAGE},
        qq{CDEF:A1=A,300,*},
        qq{CDEF:ANY1=ANY,300,*},
        qq{CDEF:AXFR1=AXFR,300,*},
        qq{CDEF:MX1=MX,300,*},
        qq{CDEF:O1=O,300,*},
        qq{CDEF:PTR1=PTR,300,*},
        qq{CDEF:SOA1=SOA,300,*},
        qq{LINE2:A1#0000FF:A},
        qq{LINE2:ANY1#FF0000:ANY},
        qq{LINE2:AXFR1#00CC00:AXFR},
        qq{LINE2:MX1#FF00FF:MX},
        qq{LINE2:O1#555555:O},
        qq{LINE2:PTR1#880000:PTR},
        qq{LINE2:SOA1#000088:SOA},
        qq{COMMENT:\\n},
        qq{GPRINT:A1:LAST:A    (cur) \\: %5.1lf },
        qq{GPRINT:A1:MAX:A    (max) \\: %5.1lf },
        qq{GPRINT:A1:MIN:A    (min) \\: %5.1lf },
        qq{GPRINT:A1:AVERAGE:A    (avg) \\: %5.1lf\\n},
        qq{GPRINT:ANY1:LAST:ANY  (cur) \\: %5.1lf },
        qq{GPRINT:ANY1:MAX:ANY  (max) \\: %5.1lf },
        qq{GPRINT:ANY1:MIN:ANY  (min) \\: %5.1lf },
        qq{GPRINT:ANY1:AVERAGE:ANY  (avg) \\: %5.1lf\\n},
        qq{GPRINT:AXFR1:LAST:AXFR (cur) \\: %5.1lf },
        qq{GPRINT:AXFR1:MAX:AXFR (max) \\: %5.1lf },
        qq{GPRINT:AXFR1:MIN:AXFR (min) \\: %5.1lf },
        qq{GPRINT:AXFR1:AVERAGE:AXFR (avg) \\: %5.1lf\\n},
        qq{GPRINT:MX1:LAST:MX   (cur) \\: %5.1lf },
        qq{GPRINT:MX1:MAX:MX   (max) \\: %5.1lf },
        qq{GPRINT:MX1:MIN:MX   (min) \\: %5.1lf },
        qq{GPRINT:MX1:AVERAGE:MX   (avg) \\: %5.1lf\\n},
        qq{GPRINT:O1:LAST:O    (cur) \\: %5.1lf },
        qq{GPRINT:O1:MAX:O    (max) \\: %5.1lf },
        qq{GPRINT:O1:MIN:O    (min) \\: %5.1lf },
        qq{GPRINT:O1:AVERAGE:O    (avg) \\: %5.1lf\\n},
        qq{GPRINT:PTR1:LAST:PTR  (cur) \\: %5.1lf },
        qq{GPRINT:PTR1:MAX:PTR  (max) \\: %5.1lf },
        qq{GPRINT:PTR1:MIN:PTR  (min) \\: %5.1lf },
        qq{GPRINT:PTR1:AVERAGE:PTR  (avg) \\: %5.1lf\\n},
        qq{GPRINT:SOA1:LAST:SOA  (cur) \\: %5.1lf },
        qq{GPRINT:SOA1:MAX:SOA  (max) \\: %5.1lf },
        qq{GPRINT:SOA1:MIN:SOA  (min) \\: %5.1lf },
        qq{GPRINT:SOA1:AVERAGE:SOA  (avg) \\: %5.1lf\\n},
      ]
    },
  apache => sub {
      my($RRD,$graph,$p,$color) = @_;

      return [
        qq{-l 0},
        qq{DEF:TA=$RRD:TA:AVERAGE},
        qq{CDEF:TA1=TA,60,*},
        qq{LINE2:TA1#0000FF:Accesses per minute\\n},
        qq{GPRINT:TA1:LAST:Current \\: %lf Accesses/min\\n},
        qq{GPRINT:TA1:MIN:Minimum \\: %lf Accesses/min\\n},
        qq{GPRINT:TA1:AVERAGE:Average \\: %lf Accesses/min\\n},
        qq{GPRINT:TA1:MAX:Maximum \\: %lf Accesses/min\\n},
      ]
    },
  apache1 => sub {
      my($RRD,$graph,$p,$color) = @_;

      return [
        qq{-l 0},
        qq{DEF:TKB=$RRD:TKB:AVERAGE},
        qq{CDEF:TKB1=TKB,1024,*,8,*},
        qq{LINE2:TKB1#FF0000:Total Traffic\\n},
        qq{GPRINT:TKB1:LAST:Current \\: %3.2lf %sbps\\n},
        qq{GPRINT:TKB1:MIN:Minimum \\: %3.2lf %sbps\\n},
        qq{GPRINT:TKB1:AVERAGE:Average \\: %3.2lf %sbps\\n},
        qq{GPRINT:TKB1:MAX:Maximum \\: %3.2lf %sbps\\n},
      ]
    },
  apache2 => sub {
      my($RRD,$graph,$p,$color) = @_;

      return [
        qq{-l 0},
        qq{DEF:IS=$RRD:IS:AVERAGE},
        qq{DEF:BS=$RRD:BS:AVERAGE},
        qq{LINE2:IS#00FF00:Idle Servers\\n},
        qq{GPRINT:IS:LAST:  cur\\: %0.0lf },
        qq{GPRINT:IS:MIN:min\\: %0.0lf },
        qq{GPRINT:IS:AVERAGE:avg\\: %lf },
        qq{GPRINT:IS:MAX:max\\: %0.0lf\\n},
        qq{LINE2:BS#FF0000:Busy Servers\\n},
        qq{GPRINT:BS:LAST:  cur\\: %0.0lf },
        qq{GPRINT:BS:MIN:min\\: %0.0lf },
        qq{GPRINT:BS:AVERAGE:avg\\: %lf },
        qq{GPRINT:BS:MAX:max\\: %0.0lf\\n},
      ]
    },
  iostat => sub {
      my($RRD,$graph,$p,$color) = @_;

      my($part) = $RRD;
      $part =~ s/^.*iostat.,/,/;
      $part =~ s/\.rrd//;
      $part =~ s/\,/\//g;

      return [
        qq{-l 0},
        qq{DEF:${p}B=$RRD:b:AVERAGE},
        qq{DEF:${p}A=$RRD:asvc_t:AVERAGE},
        qq{DEF:${p}W=$RRD:wsvc_t:AVERAGE},
        qq{CDEF:${p}t=${p}B,5,LT,${p}A,${p}W,+,UNKN,IF,1000,/},
        qq{LINE2:${p}t#$color:$part},
        qq{COMMENT:\\n},
        qq{GPRINT:${p}t:LAST:  cur\\:%3.2lf%ss},
        qq{GPRINT:${p}t:MIN:min\\:%3.2lf%ss},
        qq{GPRINT:${p}t:AVERAGE:avg\\:%3.2lf%ss},
        qq{GPRINT:${p}t:MAX:max\\:%3.2lf%ss},
        qq{COMMENT:\\n},
      ]
    },
  netstat2 => sub {
      my($RRD,$graph,$p,$color) = @_;

      return [
        qq{-l 0},
        qq{-X 0},
        qq{-Y},
        qq{DEF:tcpActiveOpens=$RRD:tcpActiveOpens:AVERAGE},
        qq{DEF:tcpPassiveOpens=$RRD:tcpPassiveOpens:AVERAGE},
        qq{DEF:tcpAttemptFails=$RRD:tcpAttemptFails:AVERAGE},
        qq{DEF:tcpEstabResets=$RRD:tcpEstabResets:AVERAGE},
        qq{LINE2:tcpActiveOpens#0000FF:TCP Active Open},
        qq{COMMENT:\\n},
        qq{GPRINT:tcpActiveOpens:LAST:  cur\\:%lf conn/s},
        qq{GPRINT:tcpActiveOpens:MIN:min\\:%lf conn/s},
        qq{GPRINT:tcpActiveOpens:AVERAGE:avg\\:%lf conn/s},
        qq{GPRINT:tcpActiveOpens:MAX:max\\:%lf conn/s},
        qq{COMMENT:\\n},
        qq{LINE2:tcpPassiveOpens#880088:TCP Passive Open},
        qq{COMMENT:\\n},
        qq{GPRINT:tcpPassiveOpens:LAST:  cur\\:%lf conn/s},
        qq{GPRINT:tcpPassiveOpens:MIN:min\\:%lf conn/s},
        qq{GPRINT:tcpPassiveOpens:AVERAGE:avg\\:%lf conn/s},
        qq{GPRINT:tcpPassiveOpens:MAX:max\\:%lf conn/s},
        qq{COMMENT:\\n},
        qq{LINE2:tcpAttemptFails#FF0000:TCP Attempts Fail},
        qq{COMMENT:\\n},
        qq{GPRINT:tcpAttemptFails:LAST:  cur\\:%lf conn/s},
        qq{GPRINT:tcpAttemptFails:MIN:min\\:%lf conn/s},
        qq{GPRINT:tcpAttemptFails:AVERAGE:avg\\:%lf conn/s},
        qq{GPRINT:tcpAttemptFails:MAX:max\\:%lf conn/s},
        qq{COMMENT:\\n},
        qq{LINE2:tcpEstabResets#88FF00:TCP Established Resets},
        qq{COMMENT:\\n},
        qq{GPRINT:tcpEstabResets:LAST:  cur\\:%lf conn/s},
        qq{GPRINT:tcpEstabResets:MIN:min\\:%lf conn/s},
        qq{GPRINT:tcpEstabResets:AVERAGE:avg\\:%lf conn/s},
        qq{GPRINT:tcpEstabResets:MAX:max\\:%lf conn/s},
        qq{COMMENT:\\n},
      ]
    },
  netstat1 => sub {
      my($RRD,$graph,$p,$color) = @_;

      return [
        qq{-l 0},
        qq{-X 0},
        qq{-Y},
        qq{DEF:udpInDatagrams=$RRD:udpInDatagrams:AVERAGE},
        qq{DEF:udpOutDatagrams=$RRD:udpOutDatagrams:AVERAGE},
        qq{DEF:udpInErrors=$RRD:udpInErrors:AVERAGE},
        qq{LINE3:udpInDatagrams#00FF00:UDP In Datagrams},
        qq{COMMENT:\\n},
        qq{GPRINT:udpInDatagrams:LAST:  cur\\:%lf pps},
        qq{GPRINT:udpInDatagrams:MIN: min\\:%lf pps},
        qq{GPRINT:udpInDatagrams:AVERAGE: avg\\:%lf pps},
        qq{GPRINT:udpInDatagrams:MAX: max\\:%lf pps},
        qq{COMMENT:\\n},
        qq{LINE1:udpOutDatagrams#0000FF:UDP Out Datagrams},
        qq{COMMENT:\\n},
        qq{GPRINT:udpOutDatagrams:LAST:  cur\\:%lf pps},
        qq{GPRINT:udpOutDatagrams:MIN: min\\:%lf pps},
        qq{GPRINT:udpOutDatagrams:AVERAGE: avg\\:%lf pps},
        qq{GPRINT:udpOutDatagrams:MAX: max\\:%lf pps},
        qq{COMMENT:\\n},
        qq{LINE2:udpInErrors#FF0000:UDP In Errors},
        qq{COMMENT:\\n},
        qq{GPRINT:udpInErrors:LAST:  cur\\:%lf pps},
        qq{GPRINT:udpInErrors:MIN: min\\:%lf pps},
        qq{GPRINT:udpInErrors:AVERAGE: avg\\:%lf pps},
        qq{GPRINT:udpInErrors:MAX: max\\:%lf pps},
        qq{COMMENT:\\n},
      ]
    },
  netstat => sub {
      my($RRD,$graph,$p,$color) = @_;

      return [
        qq{-l 0},
        qq{DEF:tcpInInorderBytes=$RRD:tcpInInorderBytes:AVERAGE},
        qq{DEF:tcpOutDataBytes=$RRD:tcpOutDataBytes:AVERAGE},
        qq{DEF:tcpRetransBytes=$RRD:tcpRetransBytes:AVERAGE},
        qq{CDEF:tcpInInorderBits=tcpInInorderBytes,8,*},
        qq{CDEF:tcpOutDataBits=tcpOutDataBytes,8,*},
        qq{CDEF:tcpRetransBits=tcpRetransBytes,8,*},
        qq{LINE3:tcpInInorderBits#00FF00:In},
        qq{COMMENT:\\n},
        qq{GPRINT:tcpInInorderBits:LAST:  cur\\:%3.2lf%sbps},
        qq{GPRINT:tcpInInorderBits:MIN:min\\:%3.2lf%sbps},
        qq{GPRINT:tcpInInorderBits:AVERAGE:avg\\:%3.2lf%sbps},
        qq{GPRINT:tcpInInorderBits:MAX:max\\:%3.2lf%sbps},
        qq{COMMENT:\\n},
        qq{LINE1:tcpOutDataBits#0000FF:Out},
        qq{COMMENT:\\n},
        qq{GPRINT:tcpOutDataBits:LAST:  cur\\:%3.2lf%sbps},
        qq{GPRINT:tcpOutDataBits:MIN:min\\:%3.2lf%sbps},
        qq{GPRINT:tcpOutDataBits:AVERAGE:avg\\:%3.2lf%sbps},
        qq{GPRINT:tcpOutDataBits:MAX:max\\:%3.2lf%sbps},
        qq{COMMENT:\\n},
        qq{LINE2:tcpRetransBits#FF0000:Retrans},
        qq{COMMENT:\\n},
        qq{GPRINT:tcpRetransBits:LAST:  cur\\:%3.2lf%sbps},
        qq{GPRINT:tcpRetransBits:MIN:min\\:%3.2lf%sbps},
        qq{GPRINT:tcpRetransBits:AVERAGE:avg\\:%3.2lf%sbps},
        qq{GPRINT:tcpRetransBits:MAX:max\\:%3.2lf%sbps\\n},
        qq{COMMENT:\\n},
      ]
    },
  netstat3 => sub {
        my($RRD,$graph,$p,$color) = @_;

        return [
          qq{-l 0},
          qq{-X 0},
          qq{-Y},
          qq{DEF:tcpInInorderPkts=$RRD:tcpInInorderPkts:AVERAGE},
          qq{DEF:tcpOutDataPkts=$RRD:tcpOutDataPkts:AVERAGE},
          qq{DEF:tcpRetransPkts=$RRD:tcpRetransPkts:AVERAGE},
          qq{LINE3:tcpInInorderPkts#00FF00:In\\n},
          qq{GPRINT:tcpInInorderPkts:LAST:  cur\\:%lf pps},
          qq{GPRINT:tcpInInorderPkts:MIN: min\\:%lf pps},
          qq{GPRINT:tcpInInorderPkts:AVERAGE: avg\\:%lf pps},
          qq{GPRINT:tcpInInorderPkts:MAX: max\\:%lf pps\\n},
          qq{LINE1:tcpOutDataPkts#0000FF:Out\\n},
          qq{GPRINT:tcpOutDataPkts:LAST:  cur\\:%lf pps},
          qq{GPRINT:tcpOutDataPkts:MIN: min\\:%lf pps},
          qq{GPRINT:tcpOutDataPkts:AVERAGE: avg\\:%lf pps},
          qq{GPRINT:tcpOutDataPkts:MAX: max\\:%lf pps\\n},
          qq{LINE2:tcpRetransPkts#FF0000:Retrans\\n},
          qq{GPRINT:tcpRetransPkts:LAST:  cur\\:%lf pps},
          qq{GPRINT:tcpRetransPkts:MIN: min\\:%lf pps},
          qq{GPRINT:tcpRetransPkts:AVERAGE: avg\\:%lf pps},
          qq{GPRINT:tcpRetransPkts:MAX: max\\:%lf pps\\n},
        ]
      },
  sendmail => sub {
      my($RRD,$graph,$p,$color) = @_;

      my($part) = $RRD;
      $part =~ s/^.*sendmail\./\,/;
      $part =~ s/\.rrd//;
      $part =~ s/\,//g;

      return [
        qq{-X 0},
        qq{-l 0},
        qq{-Y},
        qq{DEF:${p}IN=$RRD:msgsfr:AVERAGE},
        qq{CDEF:${p}IN1=${p}IN,60,*},
        qq{LINE2:${p}IN1#$color:$part IN},
        qq{GPRINT:${p}IN1:LAST:cur\\:%0.2lf m/min},
        qq{GPRINT:${p}IN1:MAX:max\\:%0.2lf m/min},
        qq{GPRINT:${p}IN1:MIN:min\\:%0.2lf m/min},
        qq{GPRINT:${p}IN1:AVERAGE:avg\\:%0.2lf m/min\\n},

        qq{DEF:${p}OUT=$RRD:msgsto:AVERAGE},
        qq{CDEF:${p}OUT1=${p}OUT,60,*},
        qq{LINE2:${p}OUT1#}.&color(\$p).qq{:$part OUT},
        qq{GPRINT:}.(--$p).qq{OUT1:LAST:cur\\:%0.2lf m/min},
        qq{GPRINT:${p}OUT1:MAX:max\\:%0.2lf m/min},
        qq{GPRINT:${p}OUT1:MIN:min\\:%0.2lf m/min},
        qq{GPRINT:${p}OUT1:AVERAGE:avg\\:%0.2lf m/min\\n},
      ]
    },
  procs => sub {
      my($RRD,$graph,$p,$color) = @_;

      my $procsrrd = $RRD;

      return [
        qq{-l 0},
        qq{DEF:p=$procsrrd:procs:AVERAGE},
        qq{AREA:p#00CC00:Processes\\n},
        qq{GPRINT:p:LAST:Current \\:%0.0lf procs\\n},
        qq{GPRINT:p:MIN:Minimum \\:%0.0lf procs\\n},
        qq{GPRINT:p:AVERAGE:Average \\:%lf procs\\n},
        qq{GPRINT:p:MAX:Maximum \\:%0.0lf procs\\n},
      ]
    },
  users => sub {
      my($RRD,$graph,$p,$color) = @_;

      my $usersrrd = $RRD;

      return [
        qq{-l 0},
        qq{DEF:u=$usersrrd:users:AVERAGE},
        qq{LINE1:u#000099:Users\\n},
        qq{GPRINT:u:LAST:Current \\:%0.0lf users\\n},
        qq{GPRINT:u:MIN:Minimum \\:%0.0lf users\\n},
        qq{GPRINT:u:AVERAGE:Average \\:%lf users\\n},
        qq{GPRINT:u:MAX:Maximum \\:%0.0lf users\\n},
      ]
    },
  ftpusers => sub {
      my($RRD,$graph,$p,$color) = @_;

      my $usersrrd = $RRD;

      return [
        qq{-l 0},
        qq{DEF:u=$usersrrd:users:AVERAGE},
        qq{LINE1:u#000099:FTP Users\\n},
        qq{GPRINT:u:LAST:Current \\:%0.0lf users\\n},
        qq{GPRINT:u:MIN:Minimum \\:%0.0lf users\\n},
        qq{GPRINT:u:AVERAGE:Average \\:%lf users\\n},
        qq{GPRINT:u:MAX:Maximum \\:%0.0lf users\\n},
      ]
    },
  files => sub {
      my($RRD,$graph,$p,$color) = @_;

      my($part) = $RRD;
      $part =~ s/^.*files,/,/;
      $part =~ s/\.rrd//;
      $part =~ s/\,/\//g;

      return [
        qq{-b 1024},
        qq{-l 0},
        qq{DEF:$p=$RRD:size:AVERAGE},
        qq{LINE2:$p#$color:$part\\n},
        qq{GPRINT:$p:LAST:  cur\\:%4.2lf%sB},
        qq{GPRINT:$p:MIN:min\\:%4.2lf%sB},
        qq{GPRINT:$p:AVERAGE:avg\\:%4.2lf%sB},
        qq{GPRINT:$p:MAX:max\\:%4.2lf%sB},
        qq{COMMENT:\\n},
      ]
    },
);

# Sanity checks on graph_titles, graph_lines, and graph_yaxes
while (my($graph,$lines) = each(%graph_lines)) {
  unless (exists($graph_titles{$graph})) {
    die("Graph name '$graph' was found in graph_lines but not in graph_titles");
  }
  unless (exists($graph_yaxes{$graph})) {
    die("Graph name '$graph' was found in graph_lines but not in graph_yaxes");
  }
}
while (my($graph,$titles) = each(%graph_titles)) {
  unless (exists($graph_lines{$graph})) {
    die("Graph name '$graph' was found in graph_titles but not in graph_lines");
  }
  unless (exists($graph_yaxes{$graph})) {
    die("Graph name '$graph' was found in graph_titles but not in graph_yaxes");
  }
}
while (my($graph,$yaxis) = each(%graph_yaxes)) {
  unless (exists($graph_lines{$graph})) {
    die("Graph name '$graph' was found in graph_yaxes but not in graph_lines");
  }
  unless (exists($graph_titles{$graph})) {
    die("Graph name '$graph' was found in graph_yaxes but not in graph_titles");
  }
}

# Created based on %graph_lines so larrd-html.pl can see what datasources
# are needed for each graph.
our %graph_dsources = ();
while (my($graph,$line_ref) = each(%graph_lines)) {
  my $def_ref = &{$line_ref}(1,'daily',1,1);
  foreach my $defline (@$def_ref) {
    if ($defline =~ /^DEF\:[^\:]+\:([^\:]+)\:AVERAGE$/) {
      my $dsource = $1;
      next unless $dsource;
      $graph_dsources{$graph}{$dsource} = 1;
    }
  }
}


sub color {
  my($p_ref,$reset) = @_;

  my(@color) = qw{0000FF FF0000 00CC00 FF00FF 555555 880000 000088 008800 008888 888888 880088 FFFF00};

  if($reset) {
    $$p_ref=0;
  }

  return $color[($$p_ref ++) % scalar(@color)];
}



#----
# &get_dsnames_old()
#
# Get DS Names from an RRD file.
# Old method using rrdinfo that loses the order.
#----

sub get_dsnames_old {
  my($rrdfile) = @_;

  my $info_ref = RRDs::info($rrdfile);

  if (RRDs::error()) {
    warn(RRDs::error());
    return;
  } else {
    my @propnames = keys(%$info_ref);
    my %dsnames = ();
    foreach my $propname (@propnames) {
      if ($propname =~ /^ds\[([^\]]+)\]\./) {
        $dsnames{$1} ++;
      }
    }
    return \%dsnames;
  }
}


#----
# &get_dsnames()
#
# Get DS Names from an RRD file.
# Return a hash but preserve the order in the
# hash values.
#----

sub get_dsnames {
  my($rrdfile) = @_;

  # Assumes that an AVERAGE consolidation function is present
  my(undef,undef,$names,undef) = RRDs::fetch($rrdfile,'AVERAGE');

  if (RRDs::error()) {
    warn(RRDs::error());
    return;
  } else {
    my $n = 0;
    my %dsnames = ();
    foreach my $name (@$names) {
      $dsnames{$name} = $n++;
    }
    return \%dsnames;
  }
}


1;

