# Contains only &larrdhtml, formerly known as larrd-html.pl

package LARRD::html;

use strict;
use Exporter;
use FileHandle;
use RRDs 1.000411;
use vars qw (@ISA @EXPORT @EXPORT_OK);

@ISA = qw(Exporter);

@EXPORT = qw(  );

@EXPORT_OK = qw( larrdhtml );

use FastGlob qw( &glob );

use lib defined($ENV{'LARRDHOME'}) ? "$ENV{'LARRDHOME'}/lib" : '/usr/local/larrd/lib';
use LARRD::graphinfo qw( %graph_dsources &get_dsnames );
use LARRD::config qw( @DATALIST @LARRDDISABLE );

##
# real work begins here
##

sub larrdhtml {
  my $html_d="$ENV{'BBHOME'}/www/html";
  my %disabled = map { $_ => 1 } @LARRDDISABLE;  # Disable Hosts Hash
  my @rrds = &glob("$ENV{'BBRRDS'}/*.rrd");
  my %services = map { $_ => 1 } @DATALIST;      # Services hash
  # we build a string with the services
  my $regexp_service = join "|", @DATALIST;
  my $date = scalar(localtime);

  my %hh;
  my %rrddb;
  foreach my $rrd (@rrds) {
    # Ignore files not updated for more than PURPLEDELAY
    if (time() - RRDs::last($rrd) > 60 * $ENV{'PURPLEDELAY'}) {
      next;
    }
    # we find $host and $service in $rrd with the previously made regexp
    # string :
    my($host, $service) = $rrd =~ /\/([^\/\.]+)\.($regexp_service).*rrd$/o;
      
    next unless $host && $service;
  
    $host =~ s/\,/\./g;
    $rrddb{$host}{$service} = $rrd;
    # we skip if $host is disabled
    next if defined $disabled{$host};
  
    # we skip if $service isn't trendable by larrd
    next unless defined $services{$service};
  
    $hh{$host}{$service}++;
  }
  
  # generate the larrd column based on %hh (host hash)
  
  foreach my $host (keys %hh) {
    print(localtime().": ".(caller(0))[3].": DEBUG $host : \n") if ($ENV{'DEBUG'});
  
    # Make the bb log to circumvent clean_string in bbd
  
    my $commahost = $host;
    $commahost =~ s/\./\,/g;

    # Here's where I need to look at the logs files that have
    # corresponding larrd files and check their colors.
  
    # Avoid possibility of BB checking the .larrd logfile while it's being created
    my $larrd_filename = "$commahost.$ENV{'LARRDCOLNAME'}";
    my $larrd_tmpfile = "$ENV{'BBTMP'}/$larrd_filename";
    my $larrd_file = "$ENV{'BBLOGS'}/$larrd_filename";
  
    my $larrdfh = new FileHandle;
    unless ($larrdfh->open(">$larrd_tmpfile")) {
      print(localtime().": ".(caller(0))[3].": ERROR Unable to open new larrd_file at '$larrd_tmpfile': $!");
      return;
    }
    $larrdfh->print("green $date - larrd is accumulating ");
    $larrdfh->print("<center><BR>");
  
    foreach my $service (@DATALIST) {
      next unless defined $hh{$host}{$service};
  
      print(localtime().": ".(caller(0))[3].": DEBUG $service ($hh{$host}{$service})\n") if ($ENV{'DEBUG'});
  
      my @subservicelist = grep { /^$service\d*/ } sort(keys(%graph_dsources));
  
      # Create a list of datasources actually in the RRD file
      my $rrdds_ref;
      unless ($rrdds_ref = &get_dsnames($rrddb{$host}{$service})) {
        next;
      }

ALLGRAPHS:
      foreach my $subservice (@subservicelist) {
        my $ds_ref = $graph_dsources{$subservice};
        # Get a list of datasources actually contained in the file
        foreach my $name (keys(%$ds_ref)) {
          unless (exists($rrdds_ref->{$name})) {
            next ALLGRAPHS;
          }
        }

        # bbnet hack that allows each service a different graph
        if ($subservice eq 'bbnet') {
          my @singlerrds = &glob("$ENV{'BBRRDS'}/$commahost.bbnet.*rrd");
          foreach my $singlerrd (@singlerrds) {
            if ($singlerrd =~ /\/$commahost\.bbnet(\.[^\.]+).*\.rrd$/) {
              my $single = $1;
              $larrdfh->print(qq{\n<p><A HREF="$ENV{'CGIBINURL'}/larrd-grapher.cgi?host=$host&service=$subservice&single=$single"><IMG SRC="$ENV{'CGIBINURL'}/larrd-grapher.cgi?host=$host&service=$subservice&single=$single&graph=hourly" ALT="larrd is accumulating $service" BORDER=0></A>\n});
            } # if ($singlerrd =~ /\/$commahost\.bbnet(\.[^\.]+).*\.rrd$/)
          } # foreach my $singlerrd (@singlerrds)
        } else {
          $larrdfh->print(qq{\n<p><A HREF="$ENV{'CGIBINURL'}/larrd-grapher.cgi?host=$host&service=$subservice"><IMG SRC="$ENV{'CGIBINURL'}/larrd-grapher.cgi?host=$host&service=$subservice&graph=hourly" ALT="larrd is accumulating $service" BORDER=0></A>\n});
        } # if ($subservice eq 'bbnet')
      } # foreach my $subservice (@subservicelist)
    } # foreach my $service (@DATALIST)
    $larrdfh->print("</center>");
    $larrdfh->close;
  
    # Time stamp it for the future
    my $time = time;
    unless (utime($time, $time + 60 * $ENV{PURPLEDELAY}, $larrd_tmpfile)) {
      print(localtime().": ".(caller(0))[3].": Unable to modify time of larrd_file $larrd_tmpfile\n");
    }
    unless (rename($larrd_tmpfile,$larrd_file)) {
      print(localtime().": ".(caller(0))[3].": Unable to rename $larrd_tmpfile to $larrd_file\n");
    }
    if ($ENV{'BBLOGSTATUS'} eq "STATIC") {
      $ENV{'QUERY_STRING'}="HOSTSVC=$commahost.$ENV{'LARRDCOLNAME'}";
      my $hostsvc = join "", `$ENV{'LARRDHOSTSVC'}`;
      unless ($hostsvc) {
        $hostsvc = q{Error: You are using Big Brother BBLOGSTATUS as STATIC.  This requires that you set the variable $ENV{'LARRDHOSTSVC'} in lib/LARRD/config.pm.  The program defined in $ENV{'LARRDHOSTSVC'}="} . $ENV{'LARRDHOSTSVC'} . q{" returned no output.  Please see your larrd-config.pl file or change BBLOGSTATUS to DYNAMIC.  This is what is recommened.};
      }
  
      $hostsvc =~ s|Content-type: text/html||;
      my $htmlfh = new FileHandle;
      unless ($htmlfh->open("> $html_d/$host.$ENV{'LARRDCOLNAME'}.html")) {
        print(localtime().": ".(caller(0))[3].": ERROR Unable to open $html_d/$host.$ENV{'LARRDCOLNAME'}.html\n");
        return;
      }
      $htmlfh->print($hostsvc);
      $htmlfh->close;
    }
  }

  return 1;
} # sub main



