#!/usr/bin/perl -w

# larrd-grapher.cgi:  The larrd cgi script to dynamically generate graphs.

# Modified by James Overbeck, grendel@gmo.jp, 2003.
# Original by:

# Scott Walters Copyright 1999
# scott@PacketPushers.com 
#
# Christian Pearce
# pearcec@PacketPushers.com
#  
#
# Do whatever you want with it
# just don't sell it and make money off of it and not cut me in ;)
# I take no liability for it, use at your own risks

require 5.006_001;

use strict;
use CGI::Pretty qw( :html );
use CGI::Carp qw(fatalsToBrowser set_message);
use FastGlob qw( &glob );
use FileHandle;
use FindBin qw($Bin $Script);
BEGIN {
  sub handle_errors {
    my $msg = shift;
    print "$0: $msg";
  }
  set_message(\&handle_errors);
}


use lib '/usr/local/larrd/lib';
use LARRD::graphinfo qw(
  %start_secs
  %timespan_names
  %graph_yaxes
  %graph_titles
  %graph_lines
  %graph_dsources
  &color
  get_dsnames
);
use LARRD::config qw( @DATALIST @LARRDDISABLE $graphwidth $graphheight);

use LARRD::butterimage qw( $buttergif );

# make install-site-perl when installing rrdtool
# use lib qq{/usr/local/rrdtool/lib/perl}; 
# Requires the most recent version, as of March 2003.
use RRDs 1.000411;

my $cgiage = (sort { $a <=> $b } (-M "$Bin/$Script", -M "/usr/local/larrd/lib/LARRD/graphinfo.pm"))[0];
   $cgiage = $cgiage * 24 * 60; # CGI Age in minutes.
                                # Turn off lazy if script was recently changed
my $date = scalar(localtime());
&main();
exit(0);

sub main {

  unless (&larrdtmp_check()) {
    die("LARRDTMP check failed");
  }
  
  open(TMP,">>$ENV{'LARRDTMP'}/larrd-grapher.debug") if $ENV{'DEBUG'};
  
  my($q) = new CGI;
  my $host = defined($q->param('host')) ? $q->param('host') : '';
  my $single = defined($q->param('single')) ? $q->param('single') : '';
  
  my $service = defined($q->param('service')) ? $q->param('service') : '';
  my $graph   = defined($q->param('graph')) ? $q->param('graph') : '';
  my $page    = defined($q->param('page')) ? $q->param('page') : '';
  my $subpage = defined($q->param('subpage')) ? $q->param('subpage') : '';

  # TODO: Changing 'cpu' to 'la' will go away eventually
  if ($service eq 'sr') { $service='vmstat9' } # RAM Scan Rate
  elsif ($service eq "cpu" ) { $service="la"; }
  elsif ($service eq "http" ) { $service="bbnet"; $single=".http"; }
  elsif ($service eq "ftp" ) { $service="bbnet"; $single=".ftp"; }
  elsif ($service eq "conn" ) { $service="bbnet"; $single=".conn"; }
  elsif ($service eq "ssh" ) { $service="bbnet"; $single=".ssh"; }
  elsif ($service eq "telnet" ) { $service="bbnet"; $single=".telnet"; }
  elsif ($service eq "nntp" ) { $service="bbnet"; $single=".nntp"; }
  elsif ($service eq "pop" ) { $service="bbnet"; $single=".pop"; }
  elsif ($service eq "pop-2" ) { $service="bbnet"; $single=".pop-2"; }
  elsif ($service eq "pop-3" ) { $service="bbnet"; $single=".pop-3"; }
  elsif ($service eq "pop2" ) { $service="bbnet"; $single=".pop2"; }
  elsif ($service eq "pop3" ) { $service="bbnet"; $single=".pop3"; }
  elsif ($service eq "smtp" ) { $service="bbnet"; $single=".smtp"; }
  elsif ($service eq "imap" ) { $service="bbnet"; $single=".imap"; }
  (my $nonumservice = $service) =~ s/\d+$//;

  # TODO: 
  # If I have an rrd file that does not contain the dsources needed to
  # make the graph that corresponds with its name, then I need to search
  # for the first numbered version of that service that does have the
  # right dsources. Since I'm talking about a single rrd file, this
  # would only happen if $host is defined to be a single host and
  # service is defined to be a single service.

  # Added this to handle the -DLARRD compile time option
  if ($service eq $ENV{'LARRDCOLNAME'}) {
    if ($graph) {
      &print_buttergif();
    } else {
      &get_host_timespan($q,$host,'hourly');
    }
  } elsif (($service) && (! exists($graph_lines{$service})) && ($host)) {
    if ($graph) {
      &print_buttergif();
    } else {
      &get_host_timespan($q,$host,'hourly');
    }
  } elsif (($service) && (! exists($graph_lines{$service}))) {
    die("No such graphable service: $service");
  } elsif ($q->param('butter')) {
    &print_buttergif();
  } elsif (($host) && ($service) && ($graph)) {
    &get_single_image($q,$host,$service,$single,$graph);
  } elsif (($host) && ($service)) {
    &get_host_service($q,$host,$service,$single);
  } elsif (($host) && ($graph)) {
    # Do larrdhtml's job: All services for a given host and timespan
    &get_host_timespan($q,$host,$graph);
  } elsif (($page) && ($subpage) && ($service) && ($graph)) {
    &get_subpage_service($q,$page,$subpage,$service,$single,$graph);
  } elsif (($page) && ($service) && ($graph)) {
    # Do I dig through all subpages or show a list of subpages or both?
    # Answer: Dig, since I'm not concerned with navigational lists yet
    &get_page_service($q,$page,$service,$single,$graph);
  } elsif (($service) && ($graph)) {
    # Dig through all from the top
    &get_top_service($q,$service,$single,$graph);
  } else {
    die("Insufficient parameters found");
  }
  return 1;
} # sub main


#----
# &print_buttergif();
#
# Print $buttergif to STDOUT along with necessary headers
#----

sub print_buttergif {
  print("Content-type: image/gif\n\n$buttergif");
}


#----
# &larrdtmp_check()
# 
# Check $ENV{'LARRDTMP'} to make sure it is writable
#----

sub larrdtmp_check {

  # Does it exist?
  if (-d $ENV{'LARRDTMP'}) {
    # Is it writable?
    unless (-w $ENV{'LARRDTMP'}) {
      # Okay it isn't. Try to use "$ENV{'BBHOME'}/www/rep/$ENV{'LARRDCOLNAME'}" instead.
      if (-d "$ENV{'BBHOME'}/www/rep/$ENV{'LARRDCOLNAME'}") {
        if (-w "$ENV{'BBHOME'}/www/rep/$ENV{'LARRDCOLNAME'}") {
          # Success! Use this.
          $ENV{'LARRDTMP'} = "$ENV{'BBHOME'}/www/rep/$ENV{'LARRDCOLNAME'}";
        } else {
          # No joy!
          die("LARRDTMP $ENV{'LARRDTMP'} isn't writable and report subdirectory $ENV{'BBHOME'}/www/rep/$ENV{'LARRDCOLNAME'} exists but isn't writable. No place to put images.");
        }
      } else {
        # warn("Failed to create larrd temp directory $ENV{'LARRDTMP'}: $!");
        if (-w "$ENV{'BBHOME'}/www/rep") {
          if (mkdir("$ENV{'BBHOME'}/www/rep/$ENV{'LARRDCOLNAME'}",0755)) {
            # Success! Use this.
            $ENV{'LARRDTMP'} = "$ENV{'BBHOME'}/www/rep/$ENV{'LARRDCOLNAME'}";
          } else {
            die("LARRDTMP $ENV{'LARRDTMP'} isn't writable and report subdirectory $ENV{'BBHOME'}/www/rep/$ENV{'LARRDCOLNAME'} couldn't be created. No place to put images.");
          } # if (mkdir("$ENV{'BBHOME'}/www/rep/$ENV{'LARRDCOLNAME'}",0755))
        } else {
          die("LARRDTMP $ENV{'LARRDTMP'} isn't writable and report directory $ENV{'BBHOME'}/www/rep isn't writable. No place to put images.");
        } # if (-w "$ENV{'BBHOME'}/www/rep")
      } # if (-d "$ENV{'BBHOME'}/www/rep/$ENV{'LARRDCOLNAME'}")
    } # unless (-w $ENV{'LARRDTMP'})
  } else {
    # Can I create it?
    unless (mkdir("$ENV{'LARRDTMP'}",0755)) {
      # Okay, try to use "$ENV{'BBHOME'}/www/rep/$ENV{'LARRDCOLNAME'}"
      if (-d "$ENV{'BBHOME'}/www/rep/$ENV{'LARRDCOLNAME'}") {
        if (-w "$ENV{'BBHOME'}/www/rep/$ENV{'LARRDCOLNAME'}") {
          # Success! Use this.
          $ENV{'LARRDTMP'} = "$ENV{'BBHOME'}/www/rep/$ENV{'LARRDCOLNAME'}";
        } else {
          # No joy!
          die("LARRDTMP $ENV{'LARRDTMP'} couldn't be created and report subdirectory $ENV{'BBHOME'}/www/rep/$ENV{'LARRDCOLNAME'} exists but isn't writable. No place to put images.");
        }
      } else {
        # warn("Failed to create larrd temp directory $ENV{'LARRDTMP'}: $!");
        if (-w "$ENV{'BBHOME'}/www/rep") {
          if (mkdir("$ENV{'BBHOME'}/www/rep/$ENV{'LARRDCOLNAME'}",0755)) {
            # Success! Use this.
            $ENV{'LARRDTMP'} = "$ENV{'BBHOME'}/www/rep/$ENV{'LARRDCOLNAME'}";
          } else {
            die("LARRDTMP $ENV{'LARRDTMP'} couldn't be created and report subdirectory $ENV{'BBHOME'}/www/rep/$ENV{'LARRDCOLNAME'} couldn't be created. No place to put images.");
          } # if (mkdir("$ENV{'BBHOME'}/www/rep/$ENV{'LARRDCOLNAME'}",0755))
        } else {
          die("LARRDTMP $ENV{'LARRDTMP'} couldn't be created and report directory $ENV{'BBHOME'}/www/rep isn't writable. No place to put images.");
        } # if (-w "$ENV{'BBHOME'}/www/rep")
      } # if (-d "$ENV{'BBHOME'}/www/rep/$ENV{'LARRDCOLNAME'}")
    } # unless (mkdir("$ENV{'LARRDTMP'}",0755))
  } # if (-d $ENV{'LARRDTMP'})

  return 1;
} # sub larrd_tmp

sub defs {
  my($host,$service,$single,$graph,$rrds_ref) = @_; 

  my(@defs) = ();
  push(@defs,'--lazy') unless $cgiage < 5; # Useful if CGI is being updated
  my($p)=0;

  &color(\$p,1);

  foreach my $RRD (@$rrds_ref) {
    print(TMP $ENV{'BBRRDS'}."/$RRD $single\n") if $ENV{'DEBUG'};
    if ($single) { unless ($RRD =~ /$single/) { next; } }
    unless (-f $ENV{'BBRRDS'}.'/'.$RRD) { die("File not found: ".$ENV{'BBRRDS'}.'/'.$RRD) }
    @defs = (@defs,@{&{$graph_lines{$service}}($ENV{'BBRRDS'}.'/'.$RRD,$graph,$p,&color(\$p) )});
  }
  print(TMP "@defs\n") if $ENV{'DEBUG'};
  return @defs;
}



#----
# &get_svccolor($host,$service,$single);
#
# Determine background color based on contents of log file
#----

sub get_svccolor {
  my($host,$service,$single) = @_;

  if ($service eq 'vmstat9') { $service = 'sr' }

  (my $nonumservice = $service) =~ s/\d+$//;
  (my $commahost = $host) =~ s/\./\,/g;
  if ($service eq 'bbnet') {
    ($service = $single) =~ s/^\.//;
    $single = '';
  } elsif ($service eq 'la') {
    $single = '';
    $service = 'cpu';
  }
  my $infh = new FileHandle;
  my $logfile = '';
  if (-f "$ENV{'BBLOGS'}/$commahost.$nonumservice") {
    $logfile = "$ENV{'BBLOGS'}/$commahost.$nonumservice";
    unless ($infh->open("< $ENV{'BBLOGS'}/$commahost.$nonumservice")) {
      warn("Unable to open log file $ENV{'BBLOGS'}/$commahost.$nonumservice");
      return 'clear';
    }
  } elsif (-f "$ENV{'BBLOGS'}/$commahost$single") {
    $logfile = "$ENV{'BBLOGS'}/$commahost$single";
    unless ($infh->open("< $ENV{'BBLOGS'}/$commahost$single")) {
      warn("Unable to open log file $ENV{'BBLOGS'}/$commahost$single");
      return 'clear';
    }
  } elsif (($nonumservice eq 'la') && (-f "$ENV{'BBLOGS'}/$commahost.cpu")) {
    $logfile = "$ENV{'BBLOGS'}/$commahost.cpu";
    unless ($infh->open("< $ENV{'BBLOGS'}/$commahost.cpu")) {
      warn("Unable to open log file $ENV{'BBLOGS'}/$commahost.cpu");
      return 'clear';
    }
  } else {
    return 'clear'; # No error per se, just no log file found
  }
  my $topline = $infh->getline();
  $infh->close;
  if ($topline =~ /^(\S+)\s+/) {
    my $color = $1;
    return $color;
  } else {
    warn("No color found in $logfile");
    return 'clear';
  }

} # get_svccolor


#---
# ($page,$subpage) = &findpage($host);
#
# Return the page and subpage for a given host.
# Try to be nice about duplicate entries that have the 'noconn' directive.
#----

sub findpage {
  my($host) = @_;

  my $bbhosts = "$ENV{'BBHOME'}/etc/bb-hosts";
  my $infh = new FileHandle;
  unless ($infh->open("< $ENV{'BBHOME'}/etc/bb-hosts")) {
    warn("Unable to open $ENV{'BBHOME'}/etc/bb-hosts");
    return;
  }
  my %pages = ();
  my %subpages = ();
  my $page = '';
  my $subpage = '';
  while (my $line = $infh->getline()) {
    chomp($line);
    next if $line =~ /^\s*\#/;
    next if $line !~ /\S/;
    $line =~ s/\s+$//; # Remove spaces at the end of the line
    if ($line =~ /^\s*page\s+(\S+)\s+/) { # Page heading
      $page = $1;
      $subpage = '';
    } elsif ($line =~ /^\s*subpage\s+(\S+)\s+/) { # subpage heading
      $subpage = $1;
    } elsif ($line =~ /^\s*(\S+)\s+(\S+)/) { # host
      my($ipaddr,$host) = ($1,$2);
      # Beware, empty strings can be keys
      if ((exists($pages{$host})) && ($line =~ /^[^\#]+\#.*noconn/)) {
        # skip it
      } else {
        $pages{$host} = $page;
      }
      if ((exists($subpages{$host})) && ($line =~ /^[^\#]+\#.*noconn/)) {
        # skip it
      } else {
        $subpages{$host} = $subpage;
      }
    } #  elsif ($line =~ /^\s*(\S+)\s+(\S+)/)
  } # while (my $line = $infh->getline())
  $infh->close;

  return($pages{$host},$subpages{$host});
}


#----
# ($page) = &findsubpage($subpage);
#
# Find the page for a given subpage
#----

sub findsubpage {
  my($subpage) = @_;

  my $bbhosts = "$ENV{'BBHOME'}/etc/bb-hosts";
  my $infh = new FileHandle;
  unless ($infh->open("< $ENV{'BBHOME'}/etc/bb-hosts")) {
    warn("Unable to open $ENV{'BBHOME'}/etc/bb-hosts");
    return;
  }
  my $page = '';
  while (my $line = $infh->getline()) {
    chomp($line);
    next if $line =~ /^\s*\#/;
    next if $line !~ /\S/;
    $line =~ s/\s+$//; # Remove spaces at the end of the line
    if ($line =~ /^\s*page\s+(\S+)\s+/) { # Page heading
      $page = $1;
      $subpage = '';
    } elsif ($line =~ /^\s*subpage\s+(\S+)\s+/) { # subpage heading
      my $bbsubpage = $1;
      if ($bbsubpage eq $subpage) {
        $infh->close;
        return $page;
      }
    }
  } # while (my $line = $infh->getline())
  $infh->close;
  return '';
} # sub findsubpage


#----
# (@subpages) = &listsubpages($page);
#
# List the subpages for a page
#----

sub listsubpages {
  my($page) = @_;
  my $bbhosts = "$ENV{'BBHOME'}/etc/bb-hosts";
  my $infh = new FileHandle;
  unless ($infh->open("< $ENV{'BBHOME'}/etc/bb-hosts")) {
    warn("Unable to open $ENV{'BBHOME'}/etc/bb-hosts");
    return;
  }
  my $bbpage = '';
  my $subpage = '';
  my @subpages = ();
  while (my $line = $infh->getline()) {
    chomp($line);
    next if $line =~ /^\s*\#/;
    $line =~ s/\#.*$//; # Remove commemts/services
    next if $line !~ /\S/;
    $line =~ s/\s+$//; # Remove spaces at the end of the line
    if ($line =~ /^\s*page\s+(\S+)\s+/) { # Page heading
      $bbpage = $1;
      $subpage = '';
    } elsif ($line =~ /^\s*subpage\s+(\S+)\s+/) { # subpage heading
      $subpage = $1;
      if ($page eq $bbpage) {
        push(@subpages,$subpage);
      }
    }
  } # while (my $line = $infh->getline())
  return @subpages;
} # sub listsubpages


#----
# (@pages) = &listpages();
#
# List the pages in bb-hosts
#----

sub listpages {
  my $bbhosts = "$ENV{'BBHOME'}/etc/bb-hosts";
  my $infh = new FileHandle;
  unless ($infh->open("< $ENV{'BBHOME'}/etc/bb-hosts")) {
    warn("Unable to open $ENV{'BBHOME'}/etc/bb-hosts");
    return;
  }
  my @pages = ();
  while (my $line = $infh->getline()) {
    chomp($line);
    next if $line =~ /^\s*\#/;
    $line =~ s/\#.*$//; # Remove commemts/services
    next if $line !~ /\S/;
    $line =~ s/\s+$//; # Remove spaces at the end of the line
    if ($line =~ /^\s*page\s+(\S+)\s+/) { # Page heading
      my $page = $1;
      push(@pages,$page);
    }
  } # while (my $line = $infh->getline())
  return @pages;
} # sub listpages


#----
# &inhale_bbhosts()
#
# Inhale bb-hosts into an impromptu database
#----

sub inhale_bbhosts {
  my %bbhosttree = ();
    # A hash of references to hashes of references to arrays.
    # Should preserve host order
  my $bbhosts = "$ENV{'BBHOME'}/etc/bb-hosts";
  my $infh = new FileHandle;
  unless ($infh->open("< $ENV{'BBHOME'}/etc/bb-hosts")) {
    warn("Unable to open $ENV{'BBHOME'}/etc/bb-hosts");
    return;
  }
  my $page = '';
  my $subpage = '';
  while (my $line = $infh->getline()) {
    chomp($line);
    next if $line =~ /^\s*\#/;
    next if $line =~ /^\s*group-compress /;
    next if $line =~ /^\s*group-only /;
    next if $line =~ /^\s*group /;
    next if $line =~ /^\s*summary /;
    next if $line =~ /^\s*dialup /;
    $line =~ s/\#.*$//; # Remove commemts/services
    next if $line !~ /\S/;
    $line =~ s/\s+$//; # Remove spaces at the end of the line
    if ($line =~ /^\s*page\s+(\S+)\s+/) { # Page heading
      $page = $1;
      $subpage = '';
    } elsif ($line =~ /^\s*subpage\s+(\S+)\s+/) { # subpage heading
      $subpage = $1;
    } elsif ($line =~ /^\s*(\S+)\s+(\S+)$/) { # host
      my($ipaddr,$host) = ($1,$2);
      push(@{$bbhosttree{$page}{$subpage}},$host); # Beware, empty strings can be keys
    }
  }
  $infh->close;
  return \%bbhosttree
}


#----
# &get_single_image($q,$host,$service,$single,$graph);
#
# Create and return a single image.
# Just saving space in &main()
#----

sub get_single_image {
  my($q,$host,$service,$single,$graph) = @_;

  (my $nonumservice = $service) =~ s/\d+$//;
  (my $commahost = $host) =~ s/\./\,/g;
  my $svccolor = &get_svccolor($host,$service,$single);
  my @colortags = ();
  if ($svccolor ne 'clear') {
    if ($svccolor eq 'purple') {
      @colortags = ('-c','SHADEA#FF00FF','-c','SHADEB#AA00AA');
    } elsif ($svccolor eq 'green') {
      @colortags = ('-c','SHADEA#00FF00','-c','SHADEB#00AA00');
    } elsif ($svccolor eq 'red') {
      @colortags = ('-c','SHADEA#FF0000','-c','SHADEB#AA0000');
    } elsif ($svccolor eq 'yellow') {
      @colortags = ('-c','SHADEA#FFFF00','-c','SHADEB#AAAA00');
    }
  } # if ($svccolor ne 'clear')
  
  $graph = "daily" if $graph !~ /minute|hourly|daily|weekly|monthly/;
     
  my($RRD_graph) = "$ENV{'LARRDTMP'}/$host.$service$single.$graph.png";
  
  # As long as vmstat3 and so on are in the vmstat RRD file, then I don't need
  # to glob, UNLESS I have some URL or a disk
  my @rrds = sort(&glob("$ENV{'BBRRDS'}/$commahost.$nonumservice$single*.rrd"));
  # If no rrds were found, print the logo.
  if ($#rrds < 0) {
    &print_buttergif();
  } else {
    foreach (@rrds) { s/^.*\/// }
    my $titlesingle = '';
    if ($single) {
      ($titlesingle = $single) =~ s/^\.//;
      if ($titlesingle =~ /conn/) {
        $titlesingle =~ s/conn/Ping/;
      } else {
        $titlesingle = uc($titlesingle);
      }
      $titlesingle .= ' ';
    }
    
    RRDs::graph($RRD_graph,"-s",$start_secs{$graph},@colortags,
      "--title","$host $titlesingle".$graph_titles{$service}.' '.$timespan_names{$graph},"-w$graphwidth","-h$graphheight",
      "-v",$graph_yaxes{$service},"-a","PNG",
      &defs($host,$service,$single,$graph,\@rrds),
      qq{COMMENT:$date\r});
    
    my($RRDerr) = RRDs::error;
    if ($RRDerr) {
      die("$RRDerr, Host: $host, Service: $service, Graph: $graph, Defs: ".join(', ',&defs($host,$service,$single,$graph,\@rrds)).", Colortags: ".join(', ',@colortags));
    }
    
    # open(GRAPH,"<$RRD_graph") or die $!;
    unless (open(GRAPH,"<$RRD_graph")) {
      unlink($RRD_graph);
      die $!;
    }
    binmode(GRAPH);
    my $pic = undef;
    my $bits = undef;
    while (my $len = read(GRAPH,$bits,8192)) {
      $pic .= $bits;
    }
    close(GRAPH);
    print $q->header(-type=>'image/png',-expires=>'+5m');
    print $pic;
  }

  return 1;
} # sub get_single_image


#----
# &print_bbheader($bbdate,$bbweb,$bbskin);
#
# Print the table header common to all BB pages
#----

sub print_bbheader {
  my($bbdate,$bbweb,$bbskin) = @_;

  print <<"EO_BBHEADER";
<TABLE SUMMARY="Navigation" WIDTH="100%"><TR><TD VALIGN=TOP ALIGN=LEFT>
<A HREF="http://bb4.com"><IMG SRC="$bbskin/bb.gif" ALT="Get Big Brother" 
        WIDTH="120" HEIGHT="24" BORDER=0></A><BR>
<A HREF="$bbweb/help/bb-ack.html"><IMG SRC="$bbskin/bbnav1.gif" 
        ALT="Contact" WIDTH="39" HEIGHT="27" BORDER=0></A>
<A HREF="$bbweb/bb2.html"><IMG SRC="$bbskin/bbnav2.gif" ALT="Condensed view" 
        WIDTH="39" HEIGHT="27" BORDER=0></A>
<A HREF="$bbweb/help/bb-rep.html"><IMG SRC="$bbskin/bbnav4.gif" ALT="Report" 
        WIDTH="39" HEIGHT="27" BORDER=0></A>
<A HREF="$bbweb/help/bb-help.html"><IMG SRC="$bbskin/bbnav3.gif" ALT="Help" 
        WIDTH="39" HEIGHT="27" BORDER=0></A>
</TD><TD VALIGN=TOP ALIGN=RIGHT>
<FONT FACE="Arial, Helvetica" SIZE=+1 COLOR="silver">
<IMG SRC="$bbskin/bbupd.gif" ALT="Last Update" WIDTH="107" HEIGHT="24"><BR>
<B>$bbdate</B></FONT></TD></TR></TABLE>
EO_BBHEADER

  return 1;
} # sub print_bbheader


#----
# &print_bbfooter();
#
# Print page footer
#----

sub print_bbfooter {
  print("<BR><IMG SRC=\"$ENV{'SCRIPT_NAME'}?butter=1\"> ");
  print qq{is proud to be built on };
  print qq{<A HREF="http://ee-staff.ethz.ch/~oetiker/webtools/rrdtool/">};
  print qq{RRDtool</A>\n};
  print("</body>\n</html>\n");
  return 1;
} # sub print_bbfooter


#----
# &get_host_service($q,$host,$service,$single);
#
# Display all the graphs timespan wise for a given host and service
#----

sub get_host_service {
  my($q,$host,$service,$single) = @_;

  (my $commahost = $host) =~ s/\./\,/g;
  (my $nonumservice = $service) =~ s/\d+$//;
  my $cgiurl = $q->url;
  my $svccolor = &get_svccolor($host,$service,$single);

  my $bgcolor = ($svccolor eq 'clear') ? 'white' : $svccolor;
  my $dispsvc = &get_dispsvc($service,$single);
  print $q->header;
  print $q->start_html(-title => "$dispsvc Trends for $host ".localtime(),
                         -BGCOLOR => "$bgcolor",
                         -BACKGROUND => "/bb/gifs/bkg-$svccolor.gif",
                         -TEXT => "#D8D8BF",
                         -LINK => "#00FFaa",
                         -VLINK => "#FFFF44");
  print qq{<center>\n};
  my $BBDATE = scalar(localtime);
  my $BBSKIN = "/bb/gifs";
  my $BBWEB = $ENV{'BBWEB'};
  &print_bbheader($BBDATE,$BBWEB,$BBSKIN);

  # Offer options for other services for this host
  my $hosturl = $ENV{'REQUEST_URI'};
     $hosturl =~ s/[\?]single\=[^\&]+\&?/\?/;
     $hosturl =~ s/[\&]single\=[^\&]+//;
     $hosturl =~ s/[\?]service\=[^\&]+\&?/\?/;
     $hosturl =~ s/[\&]service\=[^\&]+//;
     $hosturl .= '&graph=hourly';

  # Offer to list all hosts in this page/subpage for the same service
  # First need to figure out what page or subpage I'm in....
  my($page,$subpage) = &findpage($host);
  my $topurl = $ENV{'REQUEST_URI'};
     $topurl =~ s/[\?]host\=[^\&]+\&?/\?/;
     $topurl =~ s/[\&]host\=[^\&]+//;
     $topurl .= '&graph=hourly';
  my $pageurl = $topurl;
  if ($page) {
    $pageurl .= "&page=$page";
  }
  my $subpageurl = $pageurl;
  if ($subpage) {
    $subpageurl .= "&subpage=$subpage";
  }

  print("<small>");
  print qq{<A HREF="$BBWEB/bb.html">TOP</A></small><BR>};
  if ($page) {
    print qq{<A HREF="$BBWEB/$page/$page.html">}.ucfirst($page).qq{ Page</A><BR>};
  }
  if ($subpage) {
    print qq{<A HREF="$BBWEB/$page/$subpage/$subpage.html">}.ucfirst($subpage).qq{ SubPage</A><BR>};
  }
  print("</small>");
  print qq{<h1>$dispsvc Trends for $host</h1>\n};
  print("<small>");
  print qq{$dispsvc Trends: <A HREF="$topurl">All hosts</A><BR>};
  if ($page) {
    print("$dispsvc Trends Pages: <A HREF=\"$pageurl\">".ucfirst($page)."</A><BR>");
  }
  if ($subpage) {
    print("$dispsvc Trends ".ucfirst($page)." Page Subpage: <A HREF=\"$subpageurl\">".ucfirst($subpage)."</A><BR>");
  }
  print("Host $host Trends: <A HREF=\"$hosturl\">All services</A>");
  print("</small>\n");

  print("<BR><BR>\n");
  
  print("<table>\n");
  foreach my $graph ("minute","hourly","daily","weekly","monthly") {
    print qq{<tr><th>$graph</th></tr>\n};
    print qq{<tr><td><IMG SRC="$cgiurl?host=$host&service=$service&graph=$graph}.( $single ? qq{&single=$single} : '').qq{" ALT="larrd is accumulating"><BR></td></tr>\n};
  }
  print("</table>\n");
  print("</center>\n");
  
  &print_bbfooter();
  return 1;
} # sub get_host_service


#----
# &get_host_timespan($q,$host,$graph);
#
# Do larrdhtml's job for a single host and timespan
#----

sub get_host_timespan {
  my($q,$host,$graph) = @_;
  my %disabled = map { $_ => 1 } @LARRDDISABLE;  # Disable Hosts Hash, not used here
  (my $commahost = $host) =~ s/\./\,/g;
  my @rrds = &glob("$ENV{'BBRRDS'}/*.rrd");
  my $regexp_service = join "|", @DATALIST;
  my %services = map { $_ => 1 } @DATALIST;

  my %hh = ();
  my %rrddb = ();
  my %colordb = ();
  foreach my $rrd (@rrds) {
    if (time() - RRDs::last($rrd) > 60 * $ENV{'PURPLEDELAY'}) {
      next;
    }
    # Tricky.... what do I do about disk/http urls?
    if ($rrd =~ /\/([^\/\.]+)\.($regexp_service).*rrd$/) {
      my($rrdcommahost,$rrdservice) = ($1,$2);
      next unless $rrdcommahost eq $commahost;
      next if defined $disabled{$host};
      next unless defined $services{$rrdservice};
      # my $svccolor = &get_svccolor($host,$service,$single);
      if ($rrdservice eq 'bbnet') {
        if ($rrd =~ /\/[^\/\.]+\.bbnet(\.[^\.]+)\.$/) {
          my $single = $1;
          $colordb{$rrdservice.$single} = &get_svccolor($host,$rrdservice,$single);
        }
      } else {
        $colordb{$rrdservice} = &get_svccolor($host,$rrdservice,'');
      }
      $rrddb{$rrdservice} = $rrd;
      $hh{$rrdservice}++;
    }
  } # foreach my $rrd (@rrds)

  # Print HTTP and HTML headers here
  my $cgiurl = $q->url;
  my $svccolor = &color_compare(values(%colordb));
  my $bgcolor = ($svccolor eq 'clear') ? 'white' : $svccolor;
  
  print $q->header;
  print $q->start_html(-title => "trends for $host ".localtime(),
                         -BGCOLOR => "$bgcolor",
                         -BACKGROUND => "/bb/gifs/bkg-$svccolor.gif",
                         -TEXT => "#D8D8BF",
                         -LINK => "#00FFaa",
                         -VLINK => "#FFFF44");
  print qq{<center>\n};
  my $BBDATE = scalar(localtime);
  my $BBSKIN = "/bb/gifs";
  my $BBWEB = $ENV{'BBWEB'};
  &print_bbheader($BBDATE,$BBWEB,$BBSKIN);
  print("<small>");

  # Print options to display other timespans
  my($page,$subpage) = &findpage($host);
  print qq{<A HREF="$BBWEB/bb.html">TOP</A></small><BR>};
  if ($page) {
    print qq{<A HREF="$BBWEB/$page/$page.html">}.ucfirst($page).qq{ Page</A><BR>};
  }
  if ($subpage) {
    print qq{<A HREF="$BBWEB/$page/$subpage/$subpage.html">}.ucfirst($subpage).qq{ SubPage</A><BR>};
  }
  print("</small>");
  print qq{<h1>Host $host Trends</h1>};
  print("<small>");
  print("Host $host Timespans: ");
  foreach my $nextgraph ("minute","hourly","daily","weekly","monthly") {
    next if $graph eq $nextgraph;
    my $newurl = $ENV{'REQUEST_URI'};
    $newurl =~ s/graph\=$graph/graph\=$nextgraph/;
    print qq{<A HREF="$newurl">$nextgraph</A>&nbsp;};
  }
  print("</small>");

  print("<BR>");
  print("<BR>");
  print("<TABLE>");

  foreach my $service (@DATALIST) {
    next unless defined($hh{$service});
    my @subservicelist = grep { /^$service\d*/ } sort(keys(%graph_dsources));
    my $rrdds_ref;
    unless ($rrdds_ref = &get_dsnames($rrddb{$service})) {
      next;
    }
ALLGRAPHS:
    foreach my $subservice (@subservicelist) {
      my $ds_ref = $graph_dsources{$subservice};
      foreach my $name (keys(%$ds_ref)) {
        unless (exists(($rrdds_ref->{$name}))) {
          next ALLGRAPHS;
        }
      }

      # bbnet hack that allows each service a different graph
      if ($subservice eq 'bbnet') {
        my @singlerrds = &glob("$ENV{'BBRRDS'}/$commahost.bbnet.*rrd");
        foreach my $singlerrd (@singlerrds) {
          next if time() - RRDs::last($singlerrd) > 60 * $ENV{'PURPLEDELAY'};
          if ($singlerrd =~ /\/$commahost\.bbnet(\.[^\.]+).*\.rrd$/) {
            my $single = $1;
            my $dispsvc = &get_dispsvc($subservice,$single);
            print("<TR><TH>$dispsvc</TH></TR>\n");
            print(qq{<TR><TD><A HREF="$ENV{'CGIBINURL'}/larrd-grapher.cgi?host=$host&service=$subservice&single=$single"><IMG SRC="$ENV{'CGIBINURL'}/larrd-grapher.cgi?host=$host&service=$subservice&single=$single&graph=$graph" ALT="larrd is accumulating $service" BORDER=0></A></TD></TR>\n});
          } # if ($singlerrd =~ /\/$commahost\.bbnet(\.[^\.]+).*\.rrd$/)
        } # foreach my $singlerrd (@singlerrds)
      } else { # if ($subservice eq 'bbnet')
        my $dispsvc = &get_dispsvc($subservice,'');
        print("<TR><TH>$dispsvc</TH></TR>\n");
        print(qq{<TR><TD><A HREF="$ENV{'CGIBINURL'}/larrd-grapher.cgi?host=$host&service=$subservice"><IMG SRC="$ENV{'CGIBINURL'}/larrd-grapher.cgi?host=$host&service=$subservice&graph=$graph" ALT="larrd is accumulating $service" BORDER=0></A></TD></TR>\n});
      } # if ($subservice eq 'bbnet')
    } # foreach my $subservice (@subservicelist)
  } # foreach my $service (@DATALIST)
  print("</TABLE>");

  # Print HTTP and HTML footers here
  print("</center>\n");
  &print_bbfooter();
  return 1;
} # sub get_host_timespan

#----
# &color_compare(@colors);
#
# Compare a list of colors, return the worst
#----

sub color_compare {
  my(@colors) = @_;

# If any are red, return red.
  foreach my $color (@colors) {
    if ($color eq 'red') { return 'red' }
  }

# If all are clear, return clear.
  my $all_clear = 1;
  foreach my $color (@colors) {
    if ($color ne 'clear') {
      $all_clear = 0;
      last;
    }
  }
  if ($all_clear) {
    return 'clear';
  }

# If any are yellow, return yellow.
  foreach my $color (@colors) {
    if ($color eq 'yellow') {
      return 'yellow';
    }
  }

# Else, return green.
  return 'green';
} # sub color_compare



#----
# &hosts_rrd_trim(\@hostlist,$service,$single);
#
# Take a list of hosts and return only the hosts that have DS's in their RRD files to graph
# the given possibly numbered service. May return an empty list.
# Ignore any files not updated for more than an hour.
# %graph_dsources get_dsnames
#----

sub hosts_rrd_trim {
  my($hostlist_ref,$service,$single) = @_;
  (my $nonumservice = $service) =~ s/\d+$//;

  # DS's I want:
  my $graphds_ref;
  unless ($graphds_ref = $graph_dsources{$service}) {
    return; # Should always get back something here
  }

  my @newhostlist = ();
  foreach my $host (@$hostlist_ref) {
    (my $commahost = $host) =~ s/\./\,/g;
    my $searchstring = "$ENV{'BBRRDS'}/$commahost.$nonumservice$single*.rrd";
    my @rrds = &glob($searchstring);
    foreach my $rrd (@rrds) {
      next if time - RRDs::last($rrd) > 60 * $ENV{'PURPLEDELAY'};
      # DS's I have in this RRD:
      my $rrdds_ref;
      unless ($rrdds_ref = &get_dsnames($rrd)) {
        return; # I should always get something back here
      }
      my $allpresent = 1;
      while (my $dsname = each(%$graphds_ref)) {
        if (! exists($rrdds_ref->{$dsname})) {
          $allpresent = 0;
          my $resethash = scalar(keys(%$graphds_ref));
          last;
        }
      }
      if ($allpresent) {
        push(@newhostlist,$host);
        last;
      } # if ($allpresent)
    } # foreach my $rrd (@rrds)
  } # foreach my $host (@$hostlist_ref)

  return @newhostlist;
} # sub hosts_rrd_trim


#----
# &get_dispsvc($service,$single);
#
# Turn 'service' and 'single' into something suitable for small children
#----

sub get_dispsvc {
  my($service,$single) = @_;

  my $dispsvc = '';
  if ($service eq 'bbnet') {
    ($dispsvc = $single) =~ s/^\.//;
    if ($dispsvc =~ /^conn$|^ping$|^fping$/) {
      $dispsvc = " Ping Response Times";
    } else {
      $dispsvc = uc($dispsvc)." $graph_titles{'bbnet'}";
    }
  } else {
    $dispsvc = $graph_titles{$service};
  }
  return $dispsvc;
}


#----
# &get_subpage_service($q,$page,$subpage,$service,$single,$graph)
#
# Create a page for a certain service, timespan, and subpage
#----

sub get_subpage_service {
  my($q,$page,$subpage,$service,$single,$graph) = @_;

  # Subpages don't have any sub-sub pages to worry about,
  # so I'm doing this before pages or the toppage
  # Get a list of all hosts that match that subpage: bb-hosts
  my $bbtree_ref;
  unless ($bbtree_ref = &inhale_bbhosts()) {
    warn("Unable to get tree of bb-hosts");
    return;
  }
  my @hostlist = @{$bbtree_ref->{$page}{$subpage}};
  # Eliminate the ones that don't have any DS's for what I'm graphing.
  # Not enough to look at rrd file. $service can be a numbered variant.
  my @svchostlist = &hosts_rrd_trim(\@hostlist,$service,$single);
  # Here's where I need to immitate &larrdhtml().
  # Or... I need to put larrdhtml's filtering functions in an
  # equally accessible sub. Not exactly the same though because I already
  # know the service. Crap.

  # Figure out the color for each host and sevice
  my %colordb = ();
  foreach my $host (@svchostlist) {
    $colordb{$host} = &get_svccolor($host,$service,$single);
  }

  # Compare those into one single color
  my $svccolor = &color_compare(values(%colordb));

  # Make my page
  my $bgcolor = ($svccolor eq 'clear') ? 'white' : $svccolor;
  my $cgiurl = $q->url;

  my $dispsvc = &get_dispsvc($service,$single);
  print $q->header;
  print $q->start_html(-title => "$dispsvc trends - $graph".localtime(),
                       -BGCOLOR => "$bgcolor",
                       -BACKGROUND => "/bb/gifs/bkg-$svccolor.gif",
                       -TEXT => "#D8D8BF",
                       -LINK => "#00FFaa",
                       -VLINK => "#FFFF44");
  print qq{<center>\n};
  my $BBDATE = scalar(localtime);
  my $BBSKIN = "/bb/gifs";
  my $BBWEB = $ENV{'BBWEB'};
  &print_bbheader($BBDATE,$BBWEB,$BBSKIN);
  print("<small>\n");
  print qq{<A HREF="$BBWEB/bb.html">TOP</A></small>\n<BR>};
  print qq{<A HREF="$BBWEB/$page/$page.html">}.ucfirst($page).qq{ Page</A>\n<BR>};
  print qq{<A HREF="$BBWEB/$page/$subpage/$subpage.html">}.ucfirst($subpage).qq{ SubPage</A>\n};

  print("<h1>$dispsvc Trends ".ucfirst($page)." Page ".ucfirst($subpage)." SubPage</h1>");

  print("<small>\n");

  # Print option to move to top page
  my $topurl = $ENV{'REQUEST_URI'};
     $topurl =~ s/\?page\=[^\&]+/\?/;
     $topurl =~ s/\&page\=[^\&]+//;
     $topurl =~ s/\?subpage\=[^\&]+/\?/;
     $topurl =~ s/\&subpage\=[^\&]+//;
  print("$dispsvc Trends: <A HREF=\"$topurl\">All Hosts</A>\n");
  print("<BR>");

  # TODO: Improve GUI

  # Print option to move to parent page
  my $parenturl = $ENV{'REQUEST_URI'};
     $parenturl =~ s/\?subpage\=[^\&]+/\?/;
     $parenturl =~ s/\&subpage\=[^\&]+//;
  print("$dispsvc Trends Pages: <A HREF=\"$parenturl\">".ucfirst($page)."</A>\n<BR>");

  # Print options for other timespans
  print("$dispsvc Trends ".ucfirst($page)." Page ".ucfirst($subpage)." SubPage Timespans:\n");
  foreach my $nextgraph ("minute","hourly","daily","weekly","monthly") {
    next if $graph eq $nextgraph;
    my $newurl = $ENV{'REQUEST_URI'};
    $newurl =~ s/graph\=$graph/graph\=$nextgraph/;
    print qq{<A HREF="$newurl">$nextgraph</A>&nbsp;};
  }

  print("\n</small>\n");
  print("<BR><BR>\n");
  print("<table>\n");
  foreach my $host (@svchostlist) {
    my $hosturl = $ENV{'REQUEST_URI'};
       $hosturl =~ s/\?page\=[^\&]+\&?/\?/;
       $hosturl =~ s/\&page\=[^\&]+//;
       $hosturl =~ s/\?subpage\=[^\&]+\&?/\?/;
       $hosturl =~ s/\&subpage\=[^\&]+//;
       $hosturl =~ s/\?graph\=[^\&]+\&?/\?/;
       $hosturl =~ s/\&graph\=[^\&]+//;
       $hosturl .= '&host=';
       $hosturl .= $host;
    print qq{<tr><th>$host $dispsvc Trends - $graph</th></tr>\n};
    print qq{<tr><td><A HREF="$hosturl"><IMG SRC="$cgiurl?host=$host&service=$service&graph=$graph}.( $single ? qq{&single=$single} : '').qq{" ALT="larrd is accumulating" BORDER="0"></A><BR></td></tr>\n};
  } # foreach my $host (@hostlist)

  print("</table>\n");
  print("</center>\n");
  &print_bbfooter();
  return 1;
}



#----
# &get_page_service($q,$page,$service,$single,$graph)
#
# Create a page for a certain service, timespan, and subpage
#----

sub get_page_service {
  my($q,$page,$service,$single,$graph) = @_;

  # Subpages don't have any sub-sub pages to worry about,
  # so I'm doing this before pages or the toppage
  # Get a list of all hosts that match that subpage: bb-hosts
  my $bbtree_ref;
  unless ($bbtree_ref = &inhale_bbhosts()) {
    warn("Unable to get tree of bb-hosts");
    return;
  }
  my @hostlist = ();
  while (my($subpage,$subpage_ref) = each(%{$bbtree_ref->{$page}})) {
    push(@hostlist,@$subpage_ref);
  }
  # Eliminate the ones that don't have any DS's for what I'm graphing.
  # Not enough to look at rrd file. $service can be a numbered variant.
  my @svchostlist = &hosts_rrd_trim(\@hostlist,$service,$single);
  # Here's where I need to immitate &larrdhtml().
  # Or... I need to put larrdhtml's filtering functions in an
  # equally accessible sub. Not exactly the same though because I already
  # know the service. Crap.

  # Figure out the color for each host and sevice
  my %colordb = ();
  foreach my $host (@svchostlist) {
    $colordb{$host} = &get_svccolor($host,$service,$single);
  }

  # Compare those into one single color
  my $svccolor = &color_compare(values(%colordb));

  # Make my page
  my $bgcolor = ($svccolor eq 'clear') ? 'white' : $svccolor;
  my $cgiurl = $q->url;

  my $dispsvc = &get_dispsvc($service,$single);

  print $q->header;
  print $q->start_html(-title => "$dispsvc trends - $graph".localtime(),
                       -BGCOLOR => "$bgcolor",
                       -BACKGROUND => "/bb/gifs/bkg-$svccolor.gif",
                       -TEXT => "#D8D8BF",
                       -LINK => "#00FFaa",
                       -VLINK => "#FFFF44");
  print qq{<center>\n};
  my $BBDATE = scalar(localtime);
  my $BBSKIN = "/bb/gifs";
  my $BBWEB = $ENV{'BBWEB'};
  &print_bbheader($BBDATE,$BBWEB,$BBSKIN);
  print("<small>\n");
  print qq{<A HREF="$BBWEB/bb.html">TOP</A></small>\n<BR>};
  print qq{<A HREF="$BBWEB/$page/$page.html">}.ucfirst($page).qq{ Page</A>\n};
  print("</small>\n");

  print("<h1>$dispsvc Trends ".ucfirst($page)." Page</h1>\n");

  print("<small>\n");

  # Print option to move to top page
  my $topurl = $ENV{'REQUEST_URI'};
     $topurl =~ s/\?page\=[^\&]+/\?/;
     $topurl =~ s/\&page\=[^\&]+//;
  print("$dispsvc Trends: <A HREF=\"$topurl\">All Hosts</A>\n");
  print("<BR>");

  # Print options to move to sub pages
  my @subpages = &listsubpages($page);
  if ($#subpages >= 0) {
    print("$dispsvc Trends ".ucfirst($page)." Page SubPages:\n&nbsp;");
    foreach my $subpage (@subpages) {
      (my $subpageurl = $ENV{'REQUEST_URI'}) .= "&subpage=$subpage";
      print("<A HREF=\"$subpageurl\">".ucfirst($subpage)."</A>&nbsp;");
    }
    print("<BR>");
  } # if ($#subpages >= 0)

  # TODO: Improve GUI

  # Print options for other timespans
  print("$dispsvc Trends ".ucfirst($page)." Page Timespans:\n");
  foreach my $nextgraph ("minute","hourly","daily","weekly","monthly") {
    next if $graph eq $nextgraph;
    my $newurl = $ENV{'REQUEST_URI'};
    $newurl =~ s/graph\=$graph/graph\=$nextgraph/;
    print qq{<A HREF="$newurl">$nextgraph</A>&nbsp;};
  }

  print("\n</small>\n");

  print("<BR>");
  print("<BR>");
  print("\n<table>\n");
  foreach my $host (@svchostlist) {
    my $hosturl = $ENV{'REQUEST_URI'};
       $hosturl =~ s/\?page\=[^\&]+\&?/\?/;
       $hosturl =~ s/\&page\=[^\&]+//;
       $hosturl =~ s/\?graph\=[^\&]+\&?/\?/;
       $hosturl =~ s/\&graph\=[^\&]+//;
       $hosturl .= '&host=';
       $hosturl .= $host;
    print qq{<tr><th>$host $service$single - $graph</th></tr>\n};
    print qq{<tr><td><A HREF="$hosturl"><IMG SRC="$cgiurl?host=$host&service=$service&graph=$graph}.( $single ? qq{&single=$single} : '').qq{" ALT="larrd is accumulating" BORDER="0"></A><BR></td></tr>\n};
  } # foreach my $host (@hostlist)
  print("</table>\n");
  print("</center>\n");

  &print_bbfooter();
  return 1;
}



#----
# &get_top_service($q,$service,$single,$graph)
#
# Create a page for a certain service and timespan
#----

sub get_top_service {
  my($q,$service,$single,$graph) = @_;

  # Subpages don't have any sub-sub pages to worry about,
  # so I'm doing this before pages or the toppage
  # Get a list of all hosts that match that subpage: bb-hosts
  my $bbtree_ref;
  unless ($bbtree_ref = &inhale_bbhosts()) {
    warn("Unable to get tree of bb-hosts");
    return;
  }
  my @hostlist = ();
  while (my $page  = each(%$bbtree_ref)) {
    while (my($subpage,$subpage_ref) = each(%{$bbtree_ref->{$page}})) {
      push(@hostlist,@$subpage_ref);
    }
  }
  # Eliminate the ones that don't have any DS's for what I'm graphing.
  # Not enough to look at rrd file. $service can be a numbered variant.
  my @svchostlist = &hosts_rrd_trim(\@hostlist,$service,$single);
  # Here's where I need to immitate &larrdhtml().
  # Or... I need to put larrdhtml's filtering functions in an
  # equally accessible sub. Not exactly the same though because I already
  # know the service. Crap.

  # Figure out the color for each host and sevice
  my %colordb = ();
  foreach my $host (@svchostlist) {
    $colordb{$host} = &get_svccolor($host,$service,$single);
  }

  # Compare those into one single color
  my $svccolor = &color_compare(values(%colordb));

  # Make my page
  my $bgcolor = ($svccolor eq 'clear') ? 'white' : $svccolor;
  my $cgiurl = $q->url;

  print $q->header;
  print $q->start_html(-title => "$service trends - $graph".localtime(),
                       -BGCOLOR => "$bgcolor",
                       -BACKGROUND => "/bb/gifs/bkg-$svccolor.gif",
                       -TEXT => "#D8D8BF",
                       -LINK => "#00FFaa",
                       -VLINK => "#FFFF44");
  print qq{<center>\n};
  my $BBDATE = scalar(localtime);
  my $BBSKIN = "/bb/gifs";
  my $BBWEB = $ENV{'BBWEB'};
  &print_bbheader($BBDATE,$BBWEB,$BBSKIN);
  print qq{\n<small><A HREF="$BBWEB/bb.html">TOP</A></small><BR>\n};
  my @pages = &listpages();
  if ($#pages >= 0) {
    print("<small>Pages:\n");
    foreach my $page (@pages) {
      my $pageurl = "$BBWEB/$page/$page.html";
      print qq{<A HREF="$pageurl">$page</A>&nbsp;\n};
    }
    print("</small>\n");
  } # if ($#pages >= 0)

  my $dispsvc = &get_dispsvc($service,$single);
  print("<h1>$dispsvc Trends</h1>\n");

  # Print options to move to pages
  if ($#pages >= 0) {
    print("<small>$dispsvc Trends Pages:\n");
    foreach my $page (@pages) {
      (my $pageurl = $ENV{'REQUEST_URI'}) .= "&page=$page";
      print qq{<A HREF="$pageurl">$page</A>&nbsp;\n};
    }
    print("</small>\n");
  } # if ($#pages >= 0)

  # TODO: Improve GUI

  # Print options for other timespans
  print("<BR><small>$dispsvc Trends Timespans:\n");
  foreach my $nextgraph ("minute","hourly","daily","weekly","monthly") {
    next if $graph eq $nextgraph;
    my $newurl = $ENV{'REQUEST_URI'};
    $newurl =~ s/graph\=$graph/graph\=$nextgraph/;
    print qq{<A HREF="$newurl">$nextgraph</A>&nbsp;};
  }
  print("</small>\n");
  print qq{<BR><BR>\n};
  print qq{<table>};
  foreach my $host (@svchostlist) {
    my $hosturl = $ENV{'REQUEST_URI'};
       $hosturl =~ s/\?graph\=[^\&]+\&?/\?/;
       $hosturl =~ s/\&graph\=[^\&]+//;
       $hosturl .= '&host=';
       $hosturl .= $host;
    print qq{<tr><th>$host $dispsvc Trends - $graph</th></tr>\n};
    print qq{<tr><td><A HREF="$hosturl"><IMG SRC="$cgiurl?host=$host&service=$service&graph=$graph}.( $single ? qq{&single=$single} : '').qq{" ALT="larrd is accumulating" BORDER="0"></A><BR></td></tr>\n};
  } # foreach my $host (@hostlist)
  print("</table>\n");
  print("</center>\n");

  &print_bbfooter();
  return 1;
}


