#!/usr/bin/perl -w

# Original:
# $Id: monlaa.cgi,v 1.1 2000/12/18 21:12:22 pearcec Exp $

# monlaa.cgi:  The Monthly Load Average Average.

use strict;

use RRDs;

use CGI qw/:standard *table/;
use CGI::Carp qw(fatalsToBrowser set_message);
use FastGlob qw(glob);
use lib '/usr/local/larrd/lib';
use LARRD::config;
use LARRD::graphinfo qw(&get_dsnames);

BEGIN {
    sub handle_errors {
        my $msg = shift;
        print "$0: $msg";
    }
    set_message(\&handle_errors);
}

# Configurables

my $TITLE = "Load Average Average";
my $nan = "&nbsp;";

my %mon= (1 =>'Jan',
	2 =>'Feb',
	3 =>'Mar',
	4 =>'Apr',
	5 =>'May',
	6 =>'Jun',
	7 =>'Jul',
	8 =>'Aug',
	9 =>'Sep',
	10=>'Oct',
	11=>'Nov',
	12=>'Dec');

my($q) = new CGI;

my $date = localtime;
my($month, $year) = (localtime)[4,5];

$month = $month+1;
$year = $year+1900;

my $e_m = $q->param('e_m') || $month;
my $b_m = $q->param('b_m') || $e_m+1;
$b_m = "1" if $b_m eq "13";

my $e_y = $q->param('e_y') || $year;
my $b_y = $q->param('b_y') || ($b_m>$e_m ? $e_y-1 : $e_y);


# define date lists

my @monlist=();

# If the years are different we break the month list of the yearend
# we assume the begin year and end year differ by one year.
# If we want it to do more that one year add 1..12 for each year over
# one difference.

if($b_y ne $e_y) {
    @monlist = ($b_m..12,1..$e_m);
} else {
    @monlist = ($b_m..$e_m);
}

$year = $b_y;

# Build the header

my @monprint = ();

foreach (@monlist) {
    push (@monprint,$mon{$_});
}


print header; 
print start_html($TITLE), start_table({-border=>1});
print "Begin Date: $b_m-$b_y End Date: $e_m-$e_y",br;
print "\n";
print Tr([ th(["host" ,@monprint ]) ]);

foreach my $RRD (&glob("$ENV{'BBRRDS'}/*.la.rrd")) {
  (my $commahost = $RRD) =~ s/^.*\/|\.la\.rrd$//g;
  (my $host = $commahost) =~ s/\,/\./g;

  my $rrdds_ref;
  unless ($rrdds_ref = &get_dsnames($RRD)) {
    next;
  }
  my @lakeys = keys(%$rrdds_ref);
  print("\n");
  foreach my $lakey (@lakeys) {
    print("<!-- lakey: $lakey -->\n");
  }

  print("<tr align='center'>\n");
  print("<td>$host</td>\n");
  my @test;

  my $loopyear = $year;
 
  foreach my $loopmon (@monlist) {
    my $mon = $loopmon;
    # Create a next month
    my $nextmon = $mon+1;
    my $nextyear = $loopyear;

    # If we are at the years end up the year and reset the month
    $nextyear = $loopyear+1 if $nextmon eq 13;
    my $flag=$nextmon;

    $nextmon=1 if $nextmon eq 13;

    # Make the month's hold two places.
    $mon = "0$mon" if $mon < 10;
    $nextmon = "0$nextmon" if $nextmon < 10;
    my $test_ref;

    # Take only the first one for now
    ($test_ref,undef,undef) =  RRDs::graph('If no graph functions are called, the graph will not be created','-s',"$loopyear${mon}01 00:00",'-e',"$nextyear${nextmon}01 00:00-1sec","DEF:avg=$RRD:$lakeys[0]:AVERAGE","PRINT:avg:AVERAGE:'%5.1lf'");
    if (RRDs::error) {
      print("Unable to grab load average for host $host and date $year-$mon : ".RRDs::error);
      warn("Unable to grab load average for host $host and date $year-$mon : ".RRDs::error);
      next;
    }
    my @test = @$test_ref;

    $loopyear=$loopyear+1 if $flag eq 13;

    $test[0] =~ s/\s//g;        
    $test[0] =~ s/\'//g;        
    $test[0] =~ s/nan/$nan/i;
    if ($test[0] !~ /$nan/) {
      if ($lakeys[0] eq 'la') {
        $test[0] = ($test[0]/100);
      }
      $test[0] = sprintf("%1.2f",$test[0]);
      if ($lakeys[0] eq 'ProcessorTime') {
        $test[0] = "$test[0]%";
      }
    }
    print td(["$test[0]"]);
  } # foreach my $loopmon (@monlist)
  print("</tr>\n");
} # foreach my $RRD (&glob("$ENV{'BBRRDS'}/*.la.rrd"))

print end_table, end_html;
